@@ -1,3 +1,4 @@
+from unittest import result
 from .irff_np import IRFF_NP
 from ase.io import read,write
 from .molecule import molecules
@@ -586,7 +587,8 @@ class AtomDance(object):
 
   def zmat_relax(self,atoms,nbin=10,relax_step=None,
                  zmat_variable=None,zvlo=None,zvhi=None,
-                 traj='zmat.traj',relaxlog = ''):
+                 traj='zmat.traj',relaxlog = '',reset=False):
+      atoms_ = atoms.copy()
       zmatrix  = np.array(self.get_zmatrix(atoms))
       initz    = np.array(self.InitZmat)         # target
       relax_step_ = nbin if relax_step is None else relax_step
@@ -641,10 +643,11 @@ class AtomDance(object):
                    break
              nb_ += 1
       his.close()
+      if reset: images.append(atoms_)
       return images,relaxlog
 
 
-  def get_optimal_zv(self,atoms,zmat_variable,optgen,nbin=100,maxiter=100,
+  def get_optimal_zv(self,atoms,zmat_variable,optgen=None,nbin=100,maxiter=100,
                        plot_opt=False):
       i,j           = zmat_variable
       atoms_        = atoms.copy()
@@ -675,7 +678,7 @@ class AtomDance(object):
             half_  = zmat_l+(zmat_r - zmat_l)/2
             dEdz   = self.get_dz(atoms_,half_,dz,bin_)
             E.append(self.ir.E)
-            print('-  Iter %3d: ' %iter_,'%6.4f %8.4f %8.4f %8.4f' %(half_[i][j],dEdz_l,dEdz,dEdz_r))
+            #print('-  Iter %3d: ' %iter_,'%6.4f %8.4f %8.4f %8.4f' %(half_[i][j],dEdz_l,dEdz,dEdz_r))
             if dEdz_l<=0.0 and dEdz_r<=0.0:
                half_ = zmat_r 
                zmat_r= zmat_r + dx
@@ -695,7 +698,7 @@ class AtomDance(object):
                dEdz = 0.000001          # no minima point in this reigen
                iter_= maxiter           # 
             iter_ += 1
-      
+      result = half_[i][j]
       if iter_<maxiter:
          optlog += '                     ---------------------------\n'
          optlog += '                         %d-%d-%d-%d (%d,%d) \n' %(self.zmat_id[i],
@@ -722,7 +725,7 @@ class AtomDance(object):
          self.InitZmat = half_
          self.write_zmat(half_,zfile='optimal.zmat')
          atoms_ = self.zmat_to_cartation(atoms_,half_)
-         atoms_.write(optgen)
+         if optgen is not None: atoms_.write(optgen)
          del atoms_
       else:
          optlog += 'The optimal variable search failed! \n'
@@ -738,7 +741,7 @@ class AtomDance(object):
                   color='red',label='Energy')
          plt.savefig('ZVOpt.svg',transparent=True) 
          plt.close() 
-      return optlog
+      return optlog,result
 
 
   def get_dz(self,atoms,zmat,dz,bin_):
@@ -754,6 +757,21 @@ class AtomDance(object):
       dEdz   = (e_-e)/bin_
       return dEdz
       
+  def get_zmat_uncertainty(self,atoms):
+      for i,jj in enumerate(self.zmat_index):
+         if jj[0]>=0:
+            iatom = self.zmat_id[i]
+            jatom = jj[0]
+            katom = jj[1]
+            latom = jj[2]
+            log,v = self.get_optimal_zv(atoms,(i,0))
+            if v>self.InitZmat[i][0]*1.15:   
+               # print('bond {:3d} {:2s} - {:3d} {:2s}:'.format(iatom,
+               #       self.atom_name[iatom],jatom,self.atom_name[jatom]),v)
+               return (i,0),self.InitZmat[i][0]*0.9,self.InitZmat[i][0]*1.2
+            elif v<self.ir.re[iatom][jatom]*0.9:
+               return (i,0),self.InitZmat[i][0]*0.8,self.InitZmat[i][0]*1.10
+      return None,0.0,0.0
 
   def get_zmat_info(self,zmats):
       if zmats is None or len(zmats)==0:
@@ -945,7 +963,6 @@ class AtomDance(object):
              self.ir.calculate_Delta(atoms)
       return atoms
 
-
   def checkClosedAtom(self,atoms,fc):
       self.ir.calculate_Delta(atoms)
       neighbors = getNeighbor(self.natom,self.ir.r,self.ir.r_cuta,self.ir.bo0)
@@ -970,7 +987,6 @@ class AtomDance(object):
                  neighbors = getNeighbor(self.natom,self.ir.r,self.ir.r_cuta,self.ir.bo0)
       return atoms
 
-
   def bend(self,ang=None,rang=20.0,nbin=10,scale=1.2,traj='md.traj'):
       i,j,k = ang
       axis = [i,k]
@@ -982,7 +998,6 @@ class AtomDance(object):
       images = self.rotate(atms=group,axis=axis,o=axis[0],rang=rang,nbin=nbin,traj=traj,scale=scale)
       return images
 
-
   def swing_group(self,ang=None,group=None,rang=20.0,nbin=30,scale=1.2,
                   neighbors=None,traj=None):
       i,j,k = ang
@@ -1018,7 +1033,6 @@ class AtomDance(object):
                            nbin=nbin,traj=traj,scale=scale)
       return images
 
-
   def rotate(self,atms=None,axis=None,axis_vector=None,o=None,rang=20.0,
              nbin=10,traj=None,scale=1.2):
       da = 2.0*rang/nbin
@@ -1066,7 +1080,6 @@ class AtomDance(object):
       if not traj is None: his.close()
       return images
 
-
   def swing(self,ang,st=60.0,ed=180.0,nbin=50,scale=1.2,traj=None):
       da    = (ed - st)/nbin
       i,j,k = ang
@@ -1109,7 +1122,6 @@ class AtomDance(object):
       if not traj is None: his.close()
       return images
 
-
   def continous(self,atoms,dr=0.001,da=1.0,nbin=10,traj='md.traj'):
       ''' check the continous of the PES of the current configuration '''
       de = np.zeros((self.natom,3,2))
@@ -1164,7 +1176,6 @@ class AtomDance(object):
                                         relaxlog=relaxlog)
       return images,relaxlog
 
-
   def scan_pes(self,i,n,neighbor,atoms,images,r,nbin=[5],dr=[0.1]):
       if n>=len(neighbor):
          return None
@@ -1190,7 +1201,6 @@ class AtomDance(object):
              images.append(atoms_)
       return images
 
-
   def pes(self,i,atoms,neighbor=None,nbin=[5],scandr=[0.2],traj=None):
       images = []
       r      = {}
@@ -1233,7 +1243,9 @@ class AtomDance(object):
       
       if ring:
          # return None
-         ToBeMove = [j]
+         # ToBeMove = [j]
+         group_i,group_j = get_group(i,j,atoms)
+         ToBeMove = group_j
 
       bin_     = (red - rst)/nbin
       moveDirection = self.ir.vr[j][i]/self.ir.r[i][j]
