@@ -5,6 +5,14 @@ email: fengguo@lcu.edu.cn
        gfeng.alan@foxmail.com
 
 This software package is under academic license.
+  2022-08-17:
+     version 1.4.1 fix bug of the periodic boudary conditions for small unit cell.
+  2022-06-30:
+     version 1.4.0 use Di-Bij and Dj-Bij instead of Di and Dj as input vector.
+  2022-06-24:
+     version 1.3.9 add elementary message function, and implement in the GULP Program.
+  2022-06-01:
+     version 1.3.8 add f4-->fnn, and implement the f1 correction  
   2022-01-17:
      version 1.3.3  
   2021-05-31:
