@@ -36,6 +36,27 @@ class ColData(object):
                          frame=100000,dft=dft)              # get trajs for training
       return trajs_
 
+class ColRawData(object):
+  def __init__(self,max_batch=1000):
+      ''' max_batch: max number of batch
+      '''
+      self.max_batch    = max_batch   # max number in direcs to train
+
+  def __call__(self,dft='ase',batch=50,endstep=None,rawdata={}):
+      cwd           = getcwd()
+      # gen         = self.label+'.gen'
+      #self.direcs  = {}
+      # i           = startstep
+      data      = {}
+      # running     = True
+
+      for key in rawdata:
+          trajs_ = prep_data(label=key,direcs={key:rawdata[key]},
+                             split_batch=batch,max_batch=self.max_batch,
+                             frame=100000,dft=dft)              # get trajs for training
+          # print(trajs_)
+          data.update(trajs_)
+      return data
 
 if __name__ == '__main__':
    getdata = ColData(batch=100)
