@@ -109,9 +109,9 @@ def xv2xyz():
             print(elements[line.split()[1]],x,y,z,file=fxyz)
 
 
-def single_point(atoms,xcf='VDW',xca='DRSLL',basistype='DZP',
+def single_point(atoms,id=0,xcf='VDW',xca='DRSLL',basistype='DZP',
                  val={'C':4.0,'H':1.0,'O':6.0,'N':5.0,'F':7.0,'Al':3.0},
-                 cpu=4):
+                 cpu=4,**kwargs):
     if isfile('siesta.out'):
        system('rm siesta.*')
 
@@ -120,6 +120,7 @@ def single_point(atoms,xcf='VDW',xca='DRSLL',basistype='DZP',
                     xcf=xcf,xca=xca,basistype=basistype,
                     us=False) # for MD
     system('mpirun -n %d siesta<in.fdf>siesta.out' %cpu)
+    
     natom     = len(atoms)
     atom_name = atoms.get_chemical_symbols()
     spec = []
@@ -128,6 +129,8 @@ def single_point(atoms,xcf='VDW',xca='DRSLL',basistype='DZP',
            spec.append(sp)
 
     e,f,p,q = get_siesta_info(natom,spec,atom_name,val=val,label='siesta')
+    system('cp siesta.out siesta-{:d}.out'.format(id))
+    
     atoms.set_initial_charges(charges=q[-1])
     calc = SinglePointCalculator(atoms,energy=e,stress=p[-1])
     atoms.set_calculator(calc)
