@@ -5,7 +5,7 @@ from ase.io import read,write
 from ase.calculators.calculator import Calculator, all_changes
 from .qeq import qeq
 from .RadiusCutOff import setRcut
-from .reaxfflib import read_lib,write_lib
+from .reaxfflib import read_ffield,write_lib
 from .neighbors import get_neighbors,get_pangle,get_ptorsion,get_phb
 from torch.autograd import Variable
 import torch
@@ -131,6 +131,7 @@ class IRFF(Calculator):
             self.vdwnn       = False
          rcut                = j['rcut']
          rcuta               = j['rcutBond']
+         re                  = j['rEquilibrium']
          lf.close()
          self.init_bonds()
          if mol is None:
@@ -143,7 +144,7 @@ class IRFF(Calculator):
                self.emol = 0.0
       else:
          self.p,zpe_,self.spec,self.bonds,self.offd,self.angs,self.torp,self.Hbs= \
-                       read_lib(libfile=libfile,zpe=False)
+                       read_ffield(libfile=libfile,zpe=False)
          m             = None
          self.bo_layer = None
          self.emol     = 0.0
@@ -164,7 +165,7 @@ class IRFF(Calculator):
 
       self.hbshort   = hbshort
       self.hblong    = hblong
-      self.set_rcut(rcut,rcuta)
+      self.set_rcut(rcut,rcuta,re)
       self.vdwcut    = vdwcut
       self.botol     = 0.01*self.p['cutoff']
       self.atol      = self.p['acut']   # atol
@@ -230,8 +231,8 @@ class IRFF(Calculator):
       for key in P_:
           self.P[key] = torch.from_numpy(P_[key])
 
-  def set_rcut(self,rcut,rcuta): 
-      rcut_,rcuta_,re_ = setRcut(self.bonds,rcut,rcuta,None)
+  def set_rcut(self,rcut,rcuta,re): 
+      rcut_,rcuta_,re_ = setRcut(self.bonds,rcut,rcuta,re)
       self.rcut  = rcut_    ## bond order compute cutoff
       self.rcuta = rcuta_   ## angle term cutoff
 
@@ -261,8 +262,7 @@ class IRFF(Calculator):
       if self.nn:
          self.frc = 1.0
       else:
-         self.frc = torch.where(torch.logical_and(self.r<self.rcbo_tensor,self.r>0.0001),
-                    1.0,0.0)
+         self.frc = torch.where(torch.logical_and(self.r<self.rcbo_tensor,self.r>0.0001),1.0,0.0)
       self.bodiv1 = torch.div(self.r,self.P['rosi'])
       self.bopow1 = torch.pow(self.bodiv1,self.P['bo2'])
       self.eterm1 = (1.0+self.botol)*torch.exp(torch.mul(self.P['bo1'],self.bopow1))*self.frc # consist with GULP
@@ -277,9 +277,9 @@ class IRFF(Calculator):
 
       if self.nn:
          if self.BOFunction==0:
-            fsi_  = self.eterm1
-            fpi_  = self.eterm2
-            fpp_  = self.eterm3
+            fsi_  = taper(self.eterm1,rmin=self.botol,rmax=2.0*self.botol)*(self.eterm1-self.botol)
+            fpi_  = taper(self.eterm2,rmin=self.botol,rmax=2.0*self.botol)*self.eterm2
+            fpp_  = taper(self.eterm3,rmin=self.botol,rmax=2.0*self.botol)*self.eterm3
          elif self.BOFunction==1:
             fsi_  = self.f_nn('fsi',[self.eterm1],layer=self.bo_layer[1])
             fpi_  = self.f_nn('fpi',[self.eterm2],layer=self.bo_layer[1])
@@ -305,20 +305,6 @@ class IRFF(Calculator):
          self.D_si = [torch.sum(self.bop_si,1)]  
          self.D_pi = [torch.sum(self.bop_pi,1)]
          self.D_pp = [torch.sum(self.bop_pp,1)]
-       
-      # self.N    = torch.sum(torch.where(self.bop<self.p['boc2'],0.0,1.0),1)
-      # N         = self.P['val'] - self.P['valboc'] - self.N 
-      # # S         = torch.where(N>-0.00001,-0.5,0.5)
-      # # Si        = torch.unsqueeze(S,0)
-      # # Sj        = torch.unsqueeze(S,1) 
-      # # self.S    = (Si + Sj)*self.eye
-      # Ni       = torch.unsqueeze(N,0)*self.eye
-      # Nj       = torch.unsqueeze(N,1)*self.eye
-      # cond1    = self.bop<self.p['boc2']
-      # cond2    = self.bop>=self.p['boc2']
-      # self.S   = torch.where(torch.logical_or(torch.logical_and(cond1,torch.logical_and(Ni>0,Nj>0)),
-      #                                         torch.logical_and(cond2,torch.logical_or(Ni>=0,Nj>=0))),
-      #                         -0.10,0.10)
 
   def f1(self):
       Dv  = torch.unsqueeze(self.Deltap - self.P['val'],0)    # expand_dims(self.Deltap - self.P['val'],0)
@@ -359,22 +345,30 @@ class IRFF(Calculator):
       self.f1()
       self.f45()
 
-      f11    = torch.where(self.P['ovcorr']>0.0001,self.f_1,1.0)
-      f12    = torch.where(torch.logical_and(self.P['ovcorr']>0.0001,self.P['corr13']>0.0001),
+      f11    = torch.where(self.P['ovcorr']>=0.0001,self.f_1,1.0)
+      f12    = torch.where(torch.logical_and(self.P['ovcorr']>=0.0001,self.P['corr13']>=0.0001),
                          self.f_1,1.0)
       F11        = f11*f12
       F45_       = self.f_4 * self.f_5
-      F45        = torch.where(self.P['corr13']>0.0001,F45_,1.0)
+      F45        = torch.where(self.P['corr13']>=0.0001,F45_,1.0)
       self.F     = F11*F45
-      self.bo0   = self.bop*self.F  # -0.001          # consistent with GULP
+      self.bo0   = self.bop*f11*F45  # -0.001  # consistent with GULP ## here is very strange!!
 
-      self.bo    = torch.nn.functional.relu(self.bo0 - self.atol * self.eye)  # bond-order cut-off 0.001 reaxffatol
+      self.bo    = torch.nn.functional.relu(self.bo0 - self.atol) # bond-order cut-off 0.001 reaxffatol
       self.bopi  = self.bop_pi * self.F
       self.bopp  = self.bop_pp * self.F
-      self.bosi  = self.bo0 - self.bopi - self.bopp
+      self.bosi  = torch.nn.functional.relu(self.bo0 - self.bopi - self.bopp)
       self.bso   = self.P['ovun1'] * self.P['Desi'] * self.bo0
       self.Delta = torch.sum(self.bo0, 1)
 
+      self.powb  = torch.pow(self.bosi+self.safety_value,self.P['be2'])
+      self.expb  = torch.exp(torch.mul(self.P['be1'],1.0-self.powb))
+      self.sieng = self.P['Desi']*self.bosi*self.expb 
+
+      self.pieng = torch.mul(self.P['Depi'],self.bopi)
+      self.ppeng = torch.mul(self.P['Depp'],self.bopp)
+      self.esi   = self.sieng + self.pieng + self.ppeng
+
   def f_nn(self,pre,x,layer=5):
       X   = torch.unsqueeze(torch.stack(x,dim=2),2)
 
@@ -403,29 +397,7 @@ class IRFF(Calculator):
       for t in range(1,self.messages+1):
           Di        = torch.unsqueeze(self.D[t-1],0)*self.eye
           Dj        = torch.unsqueeze(self.D[t-1],1)*self.eye
-          # Dbi       = Di  - self.H[t-1]
-          # Dbj       = Dj  - self.H[t-1]
 
-          # V         = self.P['val']
-          # Ni        = torch.unsqueeze(V,0)*self.eye - Dbi
-          # Nj        = torch.unsqueeze(V,1)*self.eye - Dbj
-          # S       = np.where(N>=0.0,-0.125,0.125)
-          # Si        = torch.where(Ni>0.9,-0.125,0.125)
-          # Sj        = torch.where(Nj>0.9,-0.125,0.125)
-          # self.S    = (Si + Sj)*self.eye
-
-          # if self.MessageFunction==3:
-          #    Dbi = Di - self.H[t-1]
-          #    Dbj = Dj - self.H[t-1]
-          #    Fi  = self.f_nn('f'+str(t),[self.S,Dbj,Dbi,self.H[t-1]],layer=self.mf_layer[1])
-          #    Fj  = torch.transpose(Fi,1,0)
-          #    F   = Fi*Fj
-          #    Fsi = F[:,:,0]
-          #    Fpi = F[:,:,1]
-          #    Fpp = F[:,:,2]
-          #    self.Hsi.append(self.Hsi[t-1]*Fsi)
-          #    self.Hpi.append(self.Hpi[t-1]*Fpi)
-          #    self.Hpp.append(self.Hpp[t-1]*Fpp)
           if self.MessageFunction==1:
              Dsi_i = torch.unsqueeze(self.D_si[t-1],0)*self.eye - self.Hsi[t-1]
              Dsi_j = torch.unsqueeze(self.D_si[t-1],1)*self.eye - self.Hsi[t-1]
@@ -436,12 +408,13 @@ class IRFF(Calculator):
              Dpp_i = torch.unsqueeze(self.D_pp[t-1],0)*self.eye - self.Hpp[t-1]
              Dpp_j = torch.unsqueeze(self.D_pp[t-1],1)*self.eye - self.Hpp[t-1]
 
-             # Fi  = self.f_nn('f'+str(t),[Dsi_j,Dpi_j,Dpp_j,Dsi_i,Dpi_i,Dpp_i,self.Hsi[t-1],self.Hpi[t-1],self.Hpp[t-1]],
-             #                  layer=self.mf_layer[1])
-             Fi  = self.f_nn('f'+str(t),[Dsi_j,Dsi_i,self.Hsi[t-1],
-                                         Dpi_j,Dpi_i,self.Hpi[t-1],
-                                         Dpp_j,Dpp_i,self.Hpp[t-1]],
-                                         layer=self.mf_layer[1])
+             Dpii  = Dpi_i + Dpp_i
+             Dpij  = Dpi_j + Dpp_j
+             Fi  = self.f_nn('fm',[Dsi_i,Dpii,self.H[t-1],Dsi_j,Dpij],layer=self.mf_layer[1])
+             #  Fi  = self.f_nn('fm',[Dsi_j,Dsi_i,self.Hsi[t-1],  # +str(t)
+             #                        Dpi_j,Dpi_i,self.Hpi[t-1],
+             #                        Dpp_j,Dpp_i,self.Hpp[t-1]],
+             #                        layer=self.mf_layer[1])
              Fj  = torch.transpose(Fi,1,0)
              F   = Fi*Fj
              Fsi = F[:,:,0]
@@ -453,7 +426,7 @@ class IRFF(Calculator):
           elif self.MessageFunction==2:
              Dbi = Di - self.H[t-1]
              Dbj = Dj - self.H[t-1]
-             Fi  = self.f_nn('f'+str(t),[Dbj,Dbi,self.Hsi[t-1],self.Hpi[t-1],self.Hpp[t-1]],
+             Fi  = self.f_nn('fm',[Dbj,Dbi,self.Hsi[t-1],self.Hpi[t-1],self.Hpp[t-1]], # +str(t)
                              layer=self.mf_layer[1])
              Fj  = torch.transpose(Fi,1,0)
              F   = Fi*Fj
@@ -464,9 +437,10 @@ class IRFF(Calculator):
              self.Hpi.append(self.Hpi[t-1]*Fpi)
              self.Hpp.append(self.Hpp[t-1]*Fpp)
           elif self.MessageFunction==3:
-             Dbi   = Di  - self.H[t-1]
-             Dbj   = Dj  - self.H[t-1]
-             Fi    = self.f_nn('f'+str(t),[Dbj,Dbi,self.H[t-1]],layer=self.mf_layer[1])
+             Dbi = Di  - self.H[t-1] # torch.unsqueeze(self.P['valboc'],0) 
+             Dbj = Dj  - self.H[t-1] # torch.unsqueeze(self.P['valboc'],1)   
+             Fi  = self.f_nn('fm',[Dbj,self.H[t-1],Dbi],layer=self.mf_layer[1]) # +str(t)
+             #Fj = self.f_nn('f'+str(t),[Dbi,self.H[t-1],Dbj],layer=self.mf_layer[1])
              Fj  = torch.transpose(Fi,1,0)
              F   = Fi*Fj
              Fsi = F[:,:,0]
@@ -475,6 +449,33 @@ class IRFF(Calculator):
              self.Hsi.append(self.Hsi[t-1]*Fsi)
              self.Hpi.append(self.Hpi[t-1]*Fpi)
              self.Hpp.append(self.Hpp[t-1]*Fpp)
+          elif self.MessageFunction==4:
+             Dbi = Di  - torch.unsqueeze(self.P['val'],0)
+             Dbj = Dj  - torch.unsqueeze(self.P['val'],1)
+             Fi  = self.f_nn('fm',[Dbj,self.H[t-1],Dbi],layer=self.mf_layer[1]) # +str(t)
+             Fj  = self.f_nn('fm',[Dbi,self.H[t-1],Dbj],layer=self.mf_layer[1]) # +str(t)
+
+            #  self.f1()
+            #  f11 = self.f_1*self.f_1
+            #  F11 = torch.where(self.P['ovcorr']>=0.0001,f11,1.0)
+
+             F   = Fi*Fj #*F11
+             self.Hsi.append(self.Hsi[t-1]*F)
+             self.Hpi.append(self.Hpi[t-1]*F)
+             self.Hpp.append(self.Hpp[t-1]*F)
+          elif self.MessageFunction==5:
+             Dbi = Di  - torch.unsqueeze(self.P['val'],0) # Di  - self.H[t-1]
+             Dbj = Dj  - torch.unsqueeze(self.P['val'],1) # Dj  - self.H[t-1]
+             Fi  = self.f_nn('fm',[Dbj,self.H[t-1],Dbi],layer=self.mf_layer[1]) # +str(t)
+             Fj  = self.f_nn('fm',[Dbi,self.H[t-1],Dbj],layer=self.mf_layer[1]) # +str(t)
+             #Fj  = torch.transpose(Fi,2,0)
+             F   = Fi*Fj
+             Fsi = F[:,:,0]
+             Fpi = F[:,:,1]
+             Fpp = F[:,:,2]
+             self.Hsi.append(Fsi)
+             self.Hpi.append(Fpi)
+             self.Hpp.append(Fpp)
           else:
              raise NotImplementedError('-  Message function not supported yet!')
           self.H.append(self.Hsi[t]+self.Hpi[t]+self.Hpp[t])
@@ -500,8 +501,16 @@ class IRFF(Calculator):
       self.Dj    = torch.unsqueeze(self.Delta,1)*self.eye
       Dbi        = self.Di - self.bo0
       Dbj        = self.Dj - self.bo0
-       
-      if self.EnergyFunction==1: 
+
+      if self.EnergyFunction==0:
+         self.powb  = torch.pow(self.bosi+self.safety_value,self.P['be2'])
+         self.expb  = torch.exp(torch.mul(self.P['be1'],1.0-self.powb))
+         self.sieng = self.P['Desi']*self.bosi*self.expb 
+
+         self.pieng = torch.mul(self.P['Depi'],self.bopi)
+         self.ppeng = torch.mul(self.P['Depp'],self.bopp)
+         self.esi   = self.sieng + self.pieng + self.ppeng
+      elif self.EnergyFunction==1: 
          self.esi = self.f_nn('fe',[self.bosi,self.bopi,self.bopp],layer=self.be_layer[1])
       elif self.EnergyFunction==2:
          self.esi = self.f_nn('fe',[-self.bosi,-self.bopi,-self.bopp],layer=self.be_layer[1])
@@ -513,14 +522,18 @@ class IRFF(Calculator):
          Fj = torch.transpose(Fi,1,0)
          self.esi  = Fi*Fj*self.bo0
       elif self.EnergyFunction==5:
-         r_        = self.bodiv1
-         mors_exp1 = torch.exp(self.P['be2']*(1.0-r_))
-         mors_exp2 = torch.square(mors_exp1) 
+         # r_        = self.bodiv1
+         # mors_exp1 = torch.exp(self.P['be2']*(1.0-r_))
+         # mors_exp2 = torch.square(mors_exp1) 
 
-         mors_exp10= torch.exp(self.P['be2']*self.P['be1'])
-         mors_exp20= torch.square(mors_exp10) 
-         emorse    = 2.0*mors_exp1 - mors_exp2 + mors_exp20 - 2.0*mors_exp10
-         self.esi  = torch.nn.ReLU(emorse)
+         # mors_exp10= torch.exp(self.P['be2']*self.P['be1'])
+         # mors_exp20= torch.square(mors_exp10) 
+         # emorse    = 2.0*mors_exp1 - mors_exp2 + mors_exp20 - 2.0*mors_exp10
+         # self.esi  = torch.nn.ReLU(emorse)
+         self.sieng = torch.mul(self.P['Desi'],self.bosi)
+         self.pieng = torch.mul(self.P['Depi'],self.bopi)
+         self.ppeng = torch.mul(self.P['Depp'],self.bopp)
+         self.esi   = self.sieng + self.pieng - self.ppeng
       else:
          raise NotImplementedError('-  This method is not implimented!')
 
@@ -550,23 +563,9 @@ class IRFF(Calculator):
 
       if self.EnergyFunction>=1: # or self.EnergyFunction==2 or self.EnergyFunction==3:
          self.ebond = - self.P['Desi']*self.esi
-      # elif self.EnergyFunction==4:
-      #    self.sieng =   self.P['Desi']*self.esi*self.bosi
-      #    self.pieng =   self.P['Depi']*self.esi*self.bopi
-      #    self.ppeng =   self.P['Depp']*self.esi*self.bopp
-      #    self.ebond = - self.sieng - self.pieng - self.ppeng
       else:
-          if self.nn:
-             self.sieng = torch.mul(self.P['Desi'],self.esi)
-          else:
-             self.powb  = torch.pow(self.bosi+self.safety_value,self.P['be2'])
-             self.expb  = torch.exp(torch.mul(self.P['be1'],1.0-self.powb))
-             self.sieng = self.P['Desi']*self.bosi*self.expb 
+         self.ebond = -self.esi
 
-          self.pieng = torch.mul(self.P['Depi'],self.bopi)
-          self.ppeng = torch.mul(self.P['Depp'],self.bopp)
-          self.esi   = self.sieng + self.pieng + self.ppeng
-          self.ebond = -self.esi
       self.Ebond = 0.5*torch.sum(self.ebond)
       return self.Ebond
 
@@ -587,7 +586,7 @@ class IRFF(Calculator):
       self.Dpil    = torch.sum(torch.unsqueeze(self.Dlp,0)*(self.bopi+self.bopp),1)
       
       Delta_lp     = torch.nn.functional.relu(self.Delta_lp+1.0) -1.0
-      self.explp   = 1.0+torch.exp(-self.P['lp3']*Delta_lp)
+      self.explp   = 1.0+torch.exp(-75.0*Delta_lp) # -self.P['lp3']
       self.elone   = torch.div(self.P['lp2']*self.Delta_lp,self.explp)
       self.Elone   = torch.sum(self.elone)
 
@@ -633,11 +632,14 @@ class IRFF(Calculator):
       sbo   = self.Dpi[self.angj]
       pbo   = self.PBO[self.angj]
       rnlp  = self.nlp[self.angj]
+      self.pbo = pbo
+      self.rnlp= rnlp
       
       # if self.nn:
       #    SBO= sbo   
       # else:
       SBO= sbo - (1.0-pbo)*(dang+self.P['val8']*rnlp)  
+      self.SBO= SBO 
       
       ok    = torch.logical_and(SBO<=1.0,SBO>0.0)
       S1    = torch.where(ok,SBO,torch.full_like(SBO,0.0))         #  0< sbo < 1                  
@@ -1245,16 +1247,14 @@ class IRFF(Calculator):
   def set_m(self,m):
       self.m = {}
       if self.BOFunction==0:
-         pres = ['fe']
+         pres = ['fe','fm']
       else:
-         pres = ['fe','fsi','fpi','fpp']
+         pres = ['fe','fsi','fpi','fpp','fm']
          
       if self.vdwnn:
          pres.append('fv')
-
-      for t in range(1,self.messages+1):
-          pres.append('f'+str(t))
-
+      # for t in range(1,self.messages+1):
+      #     pres.append('fm')
       for k_ in pres:
           for k in ['wi','bi','wo','bo']:
               key = k_+k
@@ -1262,12 +1262,12 @@ class IRFF(Calculator):
               for i in range(self.natom):
                   mi_ = []
                   for j in range(self.natom):
-                      #if k_ in ['fe','fesi','fepi','fepp','fsi','fpi','fpp','fv']:
-                      bd = self.atom_name[i] + '-' + self.atom_name[j]
-                      if bd not in self.bonds:
-                         bd = self.atom_name[j] + '-' + self.atom_name[i]
-                      # else:
-                      #  bd = self.atom_name[i] 
+                      if k_ in ['fe','fsi','fpi','fpp','fv']:
+                         bd = self.atom_name[i] + '-' + self.atom_name[j]
+                         if bd not in self.bonds:
+                            bd = self.atom_name[j] + '-' + self.atom_name[i]
+                      else:
+                         bd = self.atom_name[i] 
                       key_ = key+'_'+bd
                       if key_ in m:
                          if k in ['bi','bo']:
@@ -1275,7 +1275,7 @@ class IRFF(Calculator):
                          else:
                             mi_.append(m[key_])
                   self.m[key].append(mi_)
-              self.m[key] = torch.tensor(self.m[key],dtype=torch.double)
+              self.m[key] = torch.tensor(np.array(self.m[key]),dtype=torch.double)
 
           for k in ['w','b']:
               key = k_+k
@@ -1295,12 +1295,12 @@ class IRFF(Calculator):
                   for i in range(self.natom):
                       mi_ = []
                       for j in range(self.natom):
-                          # if k_ in ['fe','fesi','fepi','fepp','fsi','fpi','fpp','fv']:
-                          bd = self.atom_name[i] + '-' + self.atom_name[j]
-                          if bd not in self.bonds:
-                             bd = self.atom_name[j] + '-' + self.atom_name[i]
-                          # else:
-                          #   bd = self.atom_name[i] 
+                          if k_ in ['fe','fsi','fpi','fpp','fv']:
+                             bd = self.atom_name[i] + '-' + self.atom_name[j]
+                             if bd not in self.bonds:
+                                bd = self.atom_name[j] + '-' + self.atom_name[i]
+                          else:
+                             bd = self.atom_name[i] 
                           key_ = key+'_'+bd
                           if key_ in m:
                              if k == 'b':
@@ -1308,7 +1308,7 @@ class IRFF(Calculator):
                              else:
                                 mi_.append(m[key+'_'+bd][l])
                       m_.append(mi_)
-                  self.m[key].append(torch.tensor(m_,dtype=torch.double))
+                  self.m[key].append(torch.tensor(np.array(m_),dtype=torch.double))
 
   def init_bonds(self):
       self.bonds,self.offd,self.angs,self.torp,self.Hbs = [],[],[],[],[]
