@@ -5,7 +5,7 @@ from ase.io import read,write
 from ase.calculators.calculator import Calculator, all_changes
 from .qeq import qeq
 from .RadiusCutOff import setRcut
-from .reaxfflib import read_lib,write_lib
+from .reaxfflib import read_ffield,write_lib
 from .neighbors import get_neighbors,get_pangle,get_ptorsion,get_phb
 
 
@@ -62,9 +62,8 @@ def relu(x):
     return np.maximum(0, x)
 
 
-
 class IRFF_NP(object):
-  '''Intelligent Machine-Learning ASE calculator'''
+  '''Intelligent ReaxFF Machine-Learning ASE calculator'''
   name = "IRFF-NP"
   implemented_properties = ["energy", "forces"] # , "stress"]
   def __init__(self,atoms=None,
@@ -112,6 +111,7 @@ class IRFF_NP(object):
             self.vdwnn       = False
          rcut                = j['rcut']
          rcuta               = j['rcutBond']
+         re                  = j['rEquilibrium']
          lf.close()
          self.init_bonds()
 
@@ -125,7 +125,7 @@ class IRFF_NP(object):
                self.emol = 0.0
       else:
          self.p,zpe_,self.spec,self.bonds,self.offd,self.Angs,self.torp,self.Hbs= \
-                       read_lib(libfile=libfile,zpe=False)
+                       read_ffield(libfile=libfile,zpe=False)
          m                = None
          self.bo_layer    = None
          self.emol        = 0.0
@@ -151,7 +151,7 @@ class IRFF_NP(object):
       self.hbtol     = self.p['hbtol']  # hbtol
       self.hbshort   = hbshort
       self.hblong    = hblong
-      self.set_rcut(rcut,rcuta)
+      self.set_rcut(rcut,rcuta,re)
       self.vdwcut    = vdwcut
       d              = np.ones([self.natom,self.natom],dtype=np.float32)
       self.d1        = np.triu(d,k=0)
@@ -228,8 +228,8 @@ class IRFF_NP(object):
       self.P.update(P_)
 
 
-  def set_rcut(self,rcut,rcuta): 
-      rcut_,rcuta_,re_ = setRcut(self.bonds,rcut,rcuta,None)
+  def set_rcut(self,rcut,rcuta,re): 
+      rcut_,rcuta_,re_ = setRcut(self.bonds,rcut,rcuta,re)
       self.rcut  = rcut_    ## bond order compute cutoff
       self.rcuta = rcuta_  ## angle term cutoff
 
@@ -278,9 +278,9 @@ class IRFF_NP(object):
 
       if self.nn:
          if self.BOFunction==0:
-            fsi_  = self.eterm1
-            fpi_  = self.eterm2
-            fpp_  = self.eterm3
+            fsi_  = taper(self.eterm1,rmin=self.botol,rmax=2.0*self.botol)*(self.eterm1-self.botol)
+            fpi_  = taper(self.eterm2,rmin=self.botol,rmax=2.0*self.botol)*self.eterm2
+            fpp_  = taper(self.eterm3,rmin=self.botol,rmax=2.0*self.botol)*self.eterm3
          elif self.BOFunction==1:
             fsi_  = self.f_nn('fsi',[self.eterm1],layer=self.bo_layer[1])
             fpi_  = self.f_nn('fpi',[self.eterm2],layer=self.bo_layer[1])
@@ -350,22 +350,30 @@ class IRFF_NP(object):
       self.f1()
       self.f45()
 
-      f11    = np.where(self.P['ovcorr']>0.0001,self.f_1,1.0)
-      f12    = np.where(np.logical_and(self.P['ovcorr']>0.0001,self.P['corr13']>0.0001),
+      f11    = np.where(self.P['ovcorr']>=0.0001,self.f_1,1.0)
+      f12    = np.where(np.logical_and(self.P['ovcorr']>=0.0001,self.P['corr13']>=0.0001),
                          self.f_1,1.0)
       self.F11   = f11*f12
       F45_       = self.f_4 * self.f_5
-      self.F45   = np.where(self.P['corr13']>0.0001,F45_,1.0)
+      self.F45   = np.where(self.P['corr13']>=0.0001,F45_,1.0)
       self.F     = self.F11*self.F45
-      self.bo0   = self.bop*self.F  # -0.001        # consistent with GULP
+      self.bo0   = self.bop*f11*self.F45 # self.F  # -0.001 # consistent with GULP ## here is very strange!!
 
       self.bo    = relu(self.bo0 - self.atol * self.eye)  # bond-order cut-off 0.001 reaxffatol
       self.bopi  = self.bop_pi * self.F
       self.bopp  = self.bop_pp * self.F
-      self.bosi  = self.bo0 - self.bopi - self.bopp
+      self.bosi  = relu(self.bo0 - self.bopi - self.bopp)
       self.bso   = self.P['ovun1'] * self.P['Desi'] * self.bo0
       self.Delta = np.sum(self.bo0, 1)
 
+      self.powb  = np.power(self.bosi+self.safety_value,self.P['be2'])
+      self.expb  = np.exp(np.multiply(self.P['be1'],1.0-self.powb))
+      self.sieng = self.P['Desi']*self.bosi*self.expb 
+
+      self.pieng = np.multiply(self.P['Depi'],self.bopi)
+      self.ppeng = np.multiply(self.P['Depp'],self.bopp)
+      self.esi   = self.sieng + self.pieng + self.ppeng
+      self.ebond = - self.esi
 
   def f_nn(self,pre,x,layer=5):
       X   = np.expand_dims(np.stack(x,axis=2),2)
@@ -380,7 +388,6 @@ class IRFF_NP(object):
       out = np.squeeze(o_)                                          # output layer
       return out
 
-
   def message_passing(self):
       self.H         = []    # hiden states (or embeding states)
       self.D         = []    # degree matrix
@@ -398,31 +405,7 @@ class IRFF_NP(object):
       for t in range(1,self.messages+1):
           Di        = np.expand_dims(self.D[t-1],axis=0)*self.eye
           Dj        = np.expand_dims(self.D[t-1],axis=1)*self.eye
-          # Dbi       = Di  - self.H[t-1]
-          # Dbj       = Dj  - self.H[t-1]
 
-          # self.N  = np.sum(np.ceil(np.where(self.bop<self.botol,0.0,self.bop)),axis=1)
-          # V         = self.P['val'] #-1.0
-          # self.Ni   = np.expand_dims(V,axis=0)*self.eye - Dbi
-          # self.Nj   = np.expand_dims(V,axis=1)*self.eye - Dbj
-          # S       = np.where(N>=0.0,-0.125,0.125)
-          # Si        = np.where(self.Ni>0.9,-0.125,0.125)
-          # Sj        = np.where(self.Nj>0.9,-0.125,0.125)
-          # self.S    = (Si + Sj)*self.eye
-          
-          # if self.MessageFunction==3:
-          #    Dbi   = Di - self.H[t-1]
-          #    Dbj   = Dj - self.H[t-1]
-          #    Fi    = self.f_nn('f'+str(t),[self.S,Dbj,Dbi,self.H[t-1]],layer=self.mf_layer[1])
-          #    Fj    = np.transpose(Fi,[1,0,2])
-          #    F     = Fi*Fj
-          #    self.F.append(F)
-          #    Fsi   = F[:,:,0]
-          #    Fpi   = F[:,:,1]
-          #    Fpp   = F[:,:,2]
-          #    self.Hsi.append(self.Hsi[t-1]*Fsi)
-          #    self.Hpi.append(self.Hpi[t-1]*Fpi)
-          #    self.Hpp.append(self.Hpp[t-1]*Fpp)
           if self.MessageFunction==1:
              Dsi_i = np.expand_dims(self.D_si[t-1],axis=0)*self.eye - self.Hsi[t-1]
              Dsi_j = np.expand_dims(self.D_si[t-1],axis=1)*self.eye - self.Hsi[t-1]
@@ -433,12 +416,12 @@ class IRFF_NP(object):
              Dpp_i = np.expand_dims(self.D_pp[t-1],axis=0)*self.eye - self.Hpp[t-1]
              Dpp_j = np.expand_dims(self.D_pp[t-1],axis=1)*self.eye - self.Hpp[t-1]
 
+             Dpii  = Dpi_i + Dpp_i
+             Dpij  = Dpi_j + Dpp_j
+
              # Fi  = self.f_nn('f'+str(t),[Dsi_j,Dpi_j,Dpp_j,Dsi_i,Dpi_i,Dpp_i,self.Hsi[t-1],self.Hpi[t-1],self.Hpp[t-1]],
              #                  layer=self.mf_layer[1])
-             Fi  = self.f_nn('f'+str(t),[Dsi_j,Dsi_i,self.Hsi[t-1],
-                                         Dpi_j,Dpi_i,self.Hpi[t-1],
-                                         Dpp_j,Dpp_i,self.Hpp[t-1]],
-                                         layer=self.mf_layer[1])
+             Fi  = self.f_nn('fm',[Dsi_i,Dpii,self.H[t-1],Dsi_j,Dpij],layer=self.mf_layer[1])
              Fj  = np.transpose(Fi,[1,0,2])
              F   = Fi*Fj
              Fsi = F[:,:,0]
@@ -451,7 +434,7 @@ class IRFF_NP(object):
           elif self.MessageFunction==2:
              Dbi   = Di  - self.H[t-1]
              Dbj   = Dj  - self.H[t-1]
-             Fi    = self.f_nn('f'+str(t),[Dbj,Dbi,self.Hsi[t-1],self.Hpi[t-1],self.Hpp[t-1]],
+             Fi    = self.f_nn('fm',[Dbj,Dbi,self.Hsi[t-1],self.Hpi[t-1],self.Hpp[t-1]], # +str(t)
                                layer=self.mf_layer[1])
              Fj    = np.transpose(Fi,[1,0,2])
              F     = Fi*Fj
@@ -463,9 +446,10 @@ class IRFF_NP(object):
              self.Hpi.append(self.Hpi[t-1]*Fpi)
              self.Hpp.append(self.Hpp[t-1]*Fpp)
           elif self.MessageFunction==3:
-             Dbi   = Di  - self.H[t-1]
-             Dbj   = Dj  - self.H[t-1]
-             Fi    = self.f_nn('f'+str(t),[Dbj,Dbi,self.H[t-1]],layer=self.mf_layer[1])
+             Dbi   = Di - self.H[t-1] # np.expand_dims(self.P['valboc'],axis=0) 
+             Dbj   = Dj - self.H[t-1] # np.expand_dims(self.P['valboc'],axis=1)  
+             Fi    = self.f_nn('fm',[Dbj,self.H[t-1],Dbi],layer=self.mf_layer[1]) # +str(t)
+             #Fj   = self.f_nn('f'+str(t),[Dbi,self.H[t-1],Dbj],layer=self.mf_layer[1])
              Fj    = np.transpose(Fi,[1,0,2])
              F     = Fi*Fj
              self.F.append(F)
@@ -475,6 +459,35 @@ class IRFF_NP(object):
              self.Hsi.append(self.Hsi[t-1]*Fsi)
              self.Hpi.append(self.Hpi[t-1]*Fpi)
              self.Hpp.append(self.Hpp[t-1]*Fpp)
+          elif self.MessageFunction==4:
+             Dbi   = Di - np.expand_dims(self.P['val'],axis=0) 
+             Dbj   = Dj - np.expand_dims(self.P['val'],axis=1)
+             Fi    = self.f_nn('fm',[Dbj,self.H[t-1],Dbi],layer=self.mf_layer[1])   # +str(t)
+             Fj    = self.f_nn('fm',[Dbi,self.H[t-1],Dbj],layer=self.mf_layer[1])   # +str(t)
+             
+            #  self.f1()
+            #  f11 = self.f_1*self.f_1
+            #  F11 = np.where(self.P['ovcorr']>=0.0001,f11,1.0)
+
+             F     = Fi*Fj #*F11     # By default p_corr13 is always True
+             self.F.append(F)
+             self.Hsi.append(self.Hsi[t-1]*F)
+             self.Hpi.append(self.Hpi[t-1]*F)
+             self.Hpp.append(self.Hpp[t-1]*F)
+          elif self.MessageFunction==5:
+             Dbi   = Di - np.expand_dims(self.P['val'],axis=0) # Di  - self.H[t-1]
+             Dbj   = Dj - np.expand_dims(self.P['val'],axis=1) # Dj  - self.H[t-1]
+             Fi    = self.f_nn('fm',[Dbj,self.H[t-1],Dbi],layer=self.mf_layer[1]) # +str(t)
+             Fj    = self.f_nn('fm',[Dbi,self.H[t-1],Dbj],layer=self.mf_layer[1]) # +str(t)
+             #Fj    = np.transpose(Fi,[2,1,0])
+             F     = Fi*Fj
+             self.F.append(F)
+             Fsi   = F[:,:,0]
+             Fpi   = F[:,:,1]
+             Fpp   = F[:,:,2]
+             self.Hsi.append(Fsi)
+             self.Hpi.append(Fpi)
+             self.Hpp.append(Fpp)
           else:
              raise NotImplementedError('-  Message function not supported yet!')
           self.H.append(self.Hsi[t]+self.Hpi[t]+self.Hpp[t])
@@ -502,7 +515,15 @@ class IRFF_NP(object):
       Dbi        = self.Di - self.bo0
       Dbj        = self.Dj - self.bo0
 
-      if self.EnergyFunction==1: 
+      if self.EnergyFunction==0:
+         self.powb  = np.power(self.bosi+self.safety_value,self.P['be2'])
+         self.expb  = np.exp(np.multiply(self.P['be1'],1.0-self.powb))
+         self.sieng = self.P['Desi']*self.bosi*self.expb 
+
+         self.pieng = np.multiply(self.P['Depi'],self.bopi)
+         self.ppeng = np.multiply(self.P['Depp'],self.bopp)
+         self.esi   = self.sieng + self.pieng + self.ppeng
+      elif self.EnergyFunction==1: 
          self.esi = self.f_nn('fe',[self.bosi,self.bopi,self.bopp],layer=self.be_layer[1])  
       elif self.EnergyFunction==2:
          self.esi = self.f_nn('fe',[-self.bosi,-self.bopi,-self.bopp],layer=self.be_layer[1]) 
@@ -513,14 +534,19 @@ class IRFF_NP(object):
          Fi = self.f_nn('fe',[Dbj,Dbi,self.bo0],layer=self.be_layer[1])
          Fj = np.transpose(Fi,[1,0])
          self.esi = Fi*Fj*self.bo0
+      # elif self.EnergyFunction==5:
+      #    r_        = self.bodiv1
+      #    mors_exp1 = np.exp(self.P['be2']*(1.0-r_))
+      #    mors_exp2 = np.square(mors_exp1) 
+      #    mors_exp10= np.exp(self.P['be2']*self.P['be1'])
+      #    mors_exp20= np.square(mors_exp10) 
+      #    emorse    = 2.0*mors_exp1 - mors_exp2 + mors_exp20 - 2.0*mors_exp10
+      #    self.esi  = np.maximum(0,emorse)
       elif self.EnergyFunction==5:
-         r_        = self.bodiv1
-         mors_exp1 = np.exp(self.P['be2']*(1.0-r_))
-         mors_exp2 = np.square(mors_exp1) 
-         mors_exp10= np.exp(self.P['be2']*self.P['be1'])
-         mors_exp20= np.square(mors_exp10) 
-         emorse    = 2.0*mors_exp1 - mors_exp2 + mors_exp20 - 2.0*mors_exp10
-         self.esi  = np.maximum(0,emorse)
+         self.sieng = np.multiply(self.P['Desi'],self.bosi)
+         self.pieng = np.multiply(self.P['Depi'],self.bopi)
+         self.ppeng = np.multiply(self.P['Depp'],self.bopp)
+         self.esi   = self.sieng + self.pieng - self.ppeng
       else:
          raise NotImplementedError('-  This method is not implimented!')
 
@@ -550,27 +576,13 @@ class IRFF_NP(object):
 
       if self.EnergyFunction>=1: # or self.EnergyFunction==2 or self.EnergyFunction==3:
          self.ebond = - self.P['Desi']*self.esi
-      # elif self.EnergyFunction==4:
-      #    self.sieng =   self.P['Desi']*self.esi*self.bosi
-      #    self.pieng =   self.P['Depi']*self.esi*self.bopi
-      #    self.ppeng =   self.P['Depp']*self.esi*self.bopp
-      #    self.ebond = - self.sieng - self.pieng - self.ppeng
       else:
-         if self.nn:
-            self.sieng = np.multiply(self.P['Desi'],self.esi)
-         else:
-            self.powb  = np.power(self.bosi+self.safety_value,self.P['be2'])
-            self.expb  = np.exp(np.multiply(self.P['be1'],1.0-self.powb))
-            self.sieng = self.P['Desi']*self.bosi*self.expb 
-
-         self.pieng = np.multiply(self.P['Depi'],self.bopi)
-         self.ppeng = np.multiply(self.P['Depp'],self.bopp)
-         self.esi   = self.sieng + self.pieng + self.ppeng
+         #if self.nn:
          self.ebond = - self.esi
+         #else:
       self.Ebond = 0.5*np.sum(self.ebond)
       return self.Ebond
 
-
   def get_elone(self):
       self.NLPOPT  = 0.5*(self.P['vale'] - self.P['val'])
       
@@ -586,11 +598,10 @@ class IRFF_NP(object):
       self.Dlp     = self.Delta - self.P['val'] - self.Delta_lp   
       self.Dpil    = np.sum(np.expand_dims(self.Dlp,axis=0)*(self.bopi+self.bopp),1)
 
-      self.explp   = 1.0+np.exp(-self.P['lp3']*self.Delta_lp)
+      self.explp   = 1.0+np.exp(-75.0*self.Delta_lp) # self.P['lp3']
       self.elone   = self.P['lp2']*self.Delta_lp/self.explp
       self.Elone   = np.sum(self.elone)
 
-
   def get_eover(self):
       self.lpcorr= self.Delta_lp/(1.0+self.P['ovun3']*np.exp(self.P['ovun4']*self.Dpil))
       self.Delta_lpcorr = self.Dv - self.lpcorr
@@ -600,7 +611,6 @@ class IRFF_NP(object):
       self.eover = self.so*self.otrm1*self.Delta_lpcorr*self.otrm2
       self.Eover = np.sum(self.eover)
 
-
   def get_eunder(self):
       self.expeu1 = np.exp(self.P['ovun6']*self.Delta_lpcorr)
       self.eu1    = sigmoid(self.P['ovun2']*self.Delta_lpcorr)
@@ -610,7 +620,6 @@ class IRFF_NP(object):
       self.eunder = -self.P['ovun5']*(1.0-self.expeu1)*self.eu1*self.eu2   
       self.Eunder = np.sum(self.eunder)
 
-
   def get_theta(self):
       Rij = self.r[self.angi,self.angj]  
       Rjk = self.r[self.angj,self.angk]  
@@ -625,18 +634,17 @@ class IRFF_NP(object):
       cos_ = np.where(self.cos_theta>0.9999999999,0.9999999999,cos_)
       self.theta     = np.arccos(cos_)
 
-
   def get_theta0(self,dang):
       self.dang  = dang
       self.sbo   = np.squeeze(self.Dpi[self.angj])
       self.pbo   = np.squeeze(self.PBO[self.angj])
-      rnlp       = np.squeeze(self.nlp[self.angj])
+      self.rnlp  = np.squeeze(self.nlp[self.angj])
 
       # if self.nn:
       #    SBO   = self.sbo 
       #    self.SBO = SBO
       # else:
-      SBO   = self.sbo - (1.0-self.pbo)*(dang+self.P['val8']*rnlp)  
+      SBO   = self.sbo - (1.0-self.pbo)*(dang+self.P['val8']*self.rnlp)  
       self.SBO = SBO
 
       ok    = np.logical_and(SBO<=1.0,SBO>0.0)
@@ -1008,7 +1016,7 @@ class IRFF_NP(object):
          lf.close()
       else:
          self.p,zpe_,self.spec,self.bonds,self.offd,self.Angs,self.torp,self.Hbs= \
-                       read_lib(libfile=libfile,zpe=True)
+                       read_ffield(libfile=libfile,zpe=True)
          m           = None
          bo_layer    = None
       self.check_tors(self.p_tor)
@@ -1265,15 +1273,15 @@ class IRFF_NP(object):
   def set_m(self,m):
       self.m = {}
       if self.BOFunction==0:
-         pres = ['fe']
+         pres = ['fe','fm']
       else:
-         pres = ['fe','fsi','fpi','fpp']
+         pres = ['fe','fsi','fpi','fpp','fm']
          
       if self.vdwnn:
          pres.append('fv')
 
-      for t in range(1,self.messages+1):
-          pres.append('f'+str(t))
+      # for t in range(1,self.messages+1):
+      #     pres.append('fm') # +str(t)
 
       for k_ in pres:
           for k in ['wi','bi','wo','bo']:
@@ -1282,12 +1290,12 @@ class IRFF_NP(object):
               for i in range(self.natom):
                   mi_ = []
                   for j in range(self.natom):
-                      # if k_ in ['fe','fesi','fepi','fepp','fsi','fpi','fpp','fv']:
-                      bd = self.atom_name[i] + '-' + self.atom_name[j]
-                      if bd not in self.bonds:
-                         bd = self.atom_name[j] + '-' + self.atom_name[i]
-                      # else:
-                      #   bd = self.atom_name[i] 
+                      if k_ in ['fe','fsi','fpi','fpp','fv']:
+                         bd = self.atom_name[i] + '-' + self.atom_name[j]
+                         if bd not in self.bonds:
+                            bd = self.atom_name[j] + '-' + self.atom_name[i]
+                      else:
+                         bd = self.atom_name[i] 
                       key_ = key+'_'+bd
                       if key_ in m:
                          if k in ['bi','bo']:
@@ -1315,12 +1323,12 @@ class IRFF_NP(object):
                   for i in range(self.natom):
                       mi_ = []
                       for j in range(self.natom):
-                          # if k_ in ['fe','fesi','fepi','fepp','fsi','fpi','fpp','fv']:
-                          bd = self.atom_name[i] + '-' + self.atom_name[j]
-                          if bd not in self.bonds:
-                             bd = self.atom_name[j] + '-' + self.atom_name[i]
-                          # else:
-                          #   bd = self.atom_name[i] 
+                          if k_ in ['fe','fsi','fpi','fpp','fv']:
+                             bd = self.atom_name[i] + '-' + self.atom_name[j]
+                             if bd not in self.bonds:
+                                bd = self.atom_name[j] + '-' + self.atom_name[i]
+                          else:
+                             bd = self.atom_name[i] 
                           key_ = key+'_'+bd
                           if key_ in m:
                              if k == 'b':
@@ -1333,7 +1341,7 @@ class IRFF_NP(object):
   def set_m_univeral(self,m,pres=['fe','fsi','fpi','fpp']):
       # pres = ['fe','fsi','fpi','fpp']   # general neural network matrix
       for t in range(1,self.messages+1):
-          pres.append('f'+str(t))
+          pres.append('fm') # +str(t)
 
       for k_ in pres:
           for k in ['wi','bi','wo','bo']:
