@@ -11,7 +11,7 @@ def get_bond_id(mol,i,j,bdlall):
     elif bdlr in bdlall:
        ind = bdlall.index(bdlr)  
     else:
-       print('-  an error case for %s.........' %bdl)
+       print('-  an error case for {:s}.........'.format(bdl))
        print(bdlall)
        ind = None
     return ind
@@ -20,7 +20,7 @@ def get_bond_id(mol,i,j,bdlall):
 class links(object):
   ''' links the bonds angles and torsions into a big graph '''
   def __init__(self,species=None,bonds=None,angs=None,
-               tors=None,hbs=None,g=False,
+               tors=None,hbs=None,# g=False,
                vdwcut=None,
                molecules=None):
       self.dft_energy = {}
@@ -30,14 +30,13 @@ class links(object):
       self.tors   = tors
       self.hbs    = hbs
       self.vdwcut = vdwcut
-      self.g      = g
+      # self.g      = g
       self.update_links(molecules)
 
       for mol in molecules:
           self.dft_energy[mol] = molecules[mol].energy_nw
       # self.histogram()
 
-
   def update_links(self,molecules):
       self.get_bond_link(molecules)
       self.get_atom_link(molecules)
@@ -45,11 +44,11 @@ class links(object):
       self.get_torsion_link(molecules)
       self.get_vdw_link(molecules)
       self.get_hb_link(molecules)
-      if self.g:
-         self.get_glink()
-
+      # if self.g:
+      #    self.get_glink()
 
   def get_atom_link(self,molecules):
+      ''' atoms are labeled by different molecules '''
       self.atomlist = {}
       self.atlab    = {}
       self.nsp      = {}
@@ -64,12 +63,12 @@ class links(object):
                  self.atlab[sp].append(lab)
           self.nsp[sp] = len(self.atlab[sp])
 
-      self.atlall = []      # bond label concated all together
+      self.atlall = []                                # bond label concated all together
       for sp in self.species:
           self.atlall.extend(self.atlab[sp])
       
-      self.dalink = []
-      # print(self.atlall)
+      self.dalink = []                                # link the atom divided by specices to a molecule 
+      # print(self.atlall)                            # mainly used by the delta
       for a in self.atom_lab:
           self.dalink.append([self.atlall.index(a)])
 
@@ -80,38 +79,36 @@ class links(object):
               if atl[0]==mol:
                  self.atomlink[mol].append([i])
 
+#   def get_glink(self):
+#       self.natom = len(self.atlall)
+#       glist = [[] for a in self.atlall]
+#       maxg  = 0
 
-  def get_glink(self):
-      self.natom = len(self.atlall)
-      glist = [[] for a in self.atlall]
-      maxg  = 0
+#       for ia,ang in enumerate(self.angall): # [key,ai,aj,ak]
+#           atl = [ang[0],ang[2]]
+#           i_  = self.atlall.index(atl)
+#           glist[i_].append(ia+1)
 
-      for ia,ang in enumerate(self.angall): # [key,ai,aj,ak]
-          atl = [ang[0],ang[2]]
-          i_  = self.atlall.index(atl)
-          glist[i_].append(ia+1)
+#       for i,a in enumerate(self.atlall):
+#           ng = len(glist[i])
+#           if ng>maxg: maxg=ng
 
-      for i,a in enumerate(self.atlall):
-          ng = len(glist[i])
-          if ng>maxg: maxg=ng
+#       self.maxg    = maxg
+#       self.glist   = np.zeros([self.natom,self.maxg,1],dtype=np.int64)
+#       self.rijlist = np.zeros([self.natom,self.maxg,1],dtype=np.int64)
+#       self.riklist = np.zeros([self.natom,self.maxg,1],dtype=np.int64)
 
-      self.maxg    = maxg
-      self.glist   = np.zeros([self.natom,self.maxg,1],dtype=np.int64)
-      self.rijlist = np.zeros([self.natom,self.maxg,1],dtype=np.int64)
-      self.riklist = np.zeros([self.natom,self.maxg,1],dtype=np.int64)
+#       for atom,gl in enumerate(glist):
+#           for ng,g in enumerate(gl):
+#               self.glist[atom][ng][0] = g  # gather theta and R is enough
+#               # print('-  info:',maxg,g,self.atlall[atom],self.angall[g-1])
+#               key,atomj,atomi,atomk = self.angall[g-1]
 
-      for atom,gl in enumerate(glist):
-          for ng,g in enumerate(gl):
-              self.glist[atom][ng][0] = g  # gather theta and R is enough
-              # print('-  info:',maxg,g,self.atlall[atom],self.angall[g-1])
-              key,atomj,atomi,atomk = self.angall[g-1]
-               
-              ij = get_bond_id(key,atomi,atomj,self.bdlall)
-              ik = get_bond_id(key,atomi,atomk,self.bdlall)
+#               ij = get_bond_id(key,atomi,atomj,self.bdlall)
+#               ik = get_bond_id(key,atomi,atomk,self.bdlall)
  
-              self.rijlist[atom][ng][0] = ij
-              self.riklist[atom][ng][0] = ik
-
+#               self.rijlist[atom][ng][0] = ij
+#               self.riklist[atom][ng][0] = ik
 
   def get_bond_link(self,molecules):
       ''' label bonds and atoms '''
@@ -143,13 +140,15 @@ class links(object):
           self.bdlab[bd] = []
           for key in molecules:  
               for i,b in enumerate(molecules[key].bond):
-                  bn = molecules[key].atom_name[b[0]]+'-'+molecules[key].atom_name[b[1]]
-                  bnr= molecules[key].atom_name[b[1]]+'-'+molecules[key].atom_name[b[0]]
+                  iatom = np.mod(b[0],molecules[key].natom)
+                  jatom = np.mod(b[1],molecules[key].natom)
+                  bn = molecules[key].atom_name[iatom]+'-'+molecules[key].atom_name[jatom]
+                  bnr= molecules[key].atom_name[jatom]+'-'+molecules[key].atom_name[iatom]
                   if bn == bd:
                      self.rbd[bd].append(molecules[key].rbd[:,i])
                      self.bdlab[bd].append([key,b[0],b[1]])
                   if bnr==bd and (not bd==bn):
-                     print('-  an error case of %s .........' %bd)
+                     print('-  an error case of {:s} .........'.format(bd))
 
       self.bdlall = []      # bond label concated all together
       for bd in self.bonds:
@@ -162,7 +161,6 @@ class links(object):
               if bl[0]==mol:
                  self.bdlink[mol].append([i])
 
-
       self.dlist = np.zeros([self.natom,self.max_nei,1],dtype=np.int64)
       self.dalist = np.zeros([self.natom,self.max_nei,1],dtype=np.int64)
       
@@ -171,9 +169,10 @@ class links(object):
           iatom = lab[1]
           for j,jatom in enumerate(molecules[mol].table[iatom]):
               # print('- bond label:',[mol,iatom,jatom])
+              jatom_ = np.mod(jatom,molecules[mol].natom)
               ind = get_bond_id(mol,iatom,jatom,self.bdlall)
-              self.dlist[i][j][0]  = ind+1
-              self.dalist[i][j][0] = self.atom_lab.index([mol,jatom]) # jatom
+              self.dlist[i][j][0]  = ind+1                              # bonds table for atom i
+              self.dalist[i][j][0] = self.atom_lab.index([mol,jatom_])  # atoms that bonded to atom i
 
       self.dilink,self.djlink = {},{}
       for bd in self.bonds:
@@ -181,9 +180,10 @@ class links(object):
           self.dilink[bd] = np.zeros([self.nbd[bd],1],dtype=np.int64)
           self.djlink[bd] = np.zeros([self.nbd[bd],1],dtype=np.int64)
           for i,b in enumerate(self.bdlab[bd]):
-              self.dilink[bd][i] = self.atom_lab.index([b[0],b[1]])
-              self.djlink[bd][i] = self.atom_lab.index([b[0],b[2]])
-
+              iatom = b[1] % molecules[b[0]].natom
+              jatom = b[2] % molecules[b[0]].natom
+              self.dilink[bd][i] = self.atom_lab.index([b[0],iatom])
+              self.djlink[bd][i] = self.atom_lab.index([b[0],jatom])
 
   def get_angle_link(self,molecules):
       self.nang      = {}
@@ -209,8 +209,11 @@ class links(object):
               iang = []
               for i,angi in enumerate(molecules[key].ang_i):
                   ai,aj,ak = molecules[key].ang_i[i],molecules[key].ang_j[i],molecules[key].ang_k[i]
-                  an = molecules[key].atom_name[ai]+'-'+molecules[key].atom_name[aj]+'-'+molecules[key].atom_name[ak]
-                  anr= molecules[key].atom_name[ak]+'-'+molecules[key].atom_name[aj]+'-'+molecules[key].atom_name[ai]
+                  ai_= ai % molecules[key].natom
+                  aj_= aj % molecules[key].natom
+                  ak_= ak % molecules[key].natom
+                  an = molecules[key].atom_name[ai_]+'-'+molecules[key].atom_name[aj_]+'-'+molecules[key].atom_name[ak_]
+                  anr= molecules[key].atom_name[ak_]+'-'+molecules[key].atom_name[aj_]+'-'+molecules[key].atom_name[ai_]
                   if an==ang or anr==ang:
                      if anr==ang:
                         ai,aj,ak = molecules[key].ang_k[i],molecules[key].ang_j[i],molecules[key].ang_i[i]
@@ -223,9 +226,9 @@ class links(object):
 
                      self.boaij[ang].append([ij+1])
                      self.boajk[ang].append([jk+1])
-                     self.dglist[ang].append([self.atom_lab.index([key,aj])])
-                     self.dgilist[ang].append([self.atom_lab.index([key,ai])])
-                     self.dgklist[ang].append([self.atom_lab.index([key,ak])])
+                     self.dglist[ang].append([self.atom_lab.index([key,aj_])])
+                     self.dgilist[ang].append([self.atom_lab.index([key,ai_])])
+                     self.dgklist[ang].append([self.atom_lab.index([key,ak_])])
               if iang:
                  self.theta[ang].extend(molecules[key].theta[iang])
           self.angall.extend(self.anglab[ang])
@@ -240,7 +243,6 @@ class links(object):
                  for i,al in enumerate(self.anglab[ang]):
                      if al[0]==mol:
                         self.anglink[mol][ang].append([i])
-  
 
   def get_torsion_link(self,molecules):
       self.ntor      = {}
@@ -268,12 +270,20 @@ class links(object):
                   atn   = molecules[key].atom_name
                   ti,tj = molecules[key].tor_i[i],molecules[key].tor_j[i]
                   tk,tl = molecules[key].tor_k[i],molecules[key].tor_l[i]
-                  tn = atn[ti]+'-'+atn[tj]+'-'+atn[tk]+'-'+atn[tl]
-                  tnr= atn[tl]+'-'+atn[tk]+'-'+atn[tj]+'-'+atn[ti]
+                  ti_   = ti % molecules[key].natom
+                  tj_   = tj % molecules[key].natom
+                  tk_   = tk % molecules[key].natom
+                  tl_   = tl % molecules[key].natom
+                  tn    = atn[ti_]+'-'+atn[tj_]+'-'+atn[tk_]+'-'+atn[tl_]
+                  tnr   = atn[tl_]+'-'+atn[tk_]+'-'+atn[tj_]+'-'+atn[ti_]
                   if tn==tor or tnr==tor:
                      if tnr==tor and tn!=tor:
                         ti,tj = molecules[key].tor_l[i],molecules[key].tor_k[i]
                         tk,tl = molecules[key].tor_j[i],molecules[key].tor_i[i]
+                        ti_   = ti % molecules[key].natom
+                        tj_   = tj % molecules[key].natom
+                        tk_   = tk % molecules[key].natom
+                        tl_   = tl % molecules[key].natom
                      self.torlab[tor].append([key,ti,tj,tk,tl])
                      
                      itor.append(i)
@@ -286,8 +296,8 @@ class links(object):
                      self.tjk[tor].append([jk+1])
                      self.tkl[tor].append([kl+1])
 
-                     self.dtj[tor].append([self.atom_lab.index([key,tj])])
-                     self.dtk[tor].append([self.atom_lab.index([key,tk])])
+                     self.dtj[tor].append([self.atom_lab.index([key,tj_])])
+                     self.dtk[tor].append([self.atom_lab.index([key,tk_])])
               if itor:
                  self.s_ijk[tor].extend(molecules[key].s_ijk[itor])
                  self.s_jkl[tor].extend(molecules[key].s_jkl[itor])
