@@ -1,11 +1,13 @@
 from os import system
 import numpy as np
+import json as js
 from ase import Atoms
 from ase.io import read # ,write
 from ase.io.trajectory import TrajectoryWriter
 from ase.calculators.singlepoint import SinglePointCalculator
 import matplotlib.pyplot as plt
 from ..molecule import press_mol
+from ..reaxfflib import write_lib
 
 input_sample = '''
 md qiterative conv;
@@ -32,7 +34,9 @@ def write_gulp_in(A,runword='gradient qiterative nosymmetry conv verb debu',
                   time_step=0.1,
                   tot_step=10.0,
                   maxcyc=2000,
-                  lib='reax'):
+                  output=None,
+                  restart=100,
+                  lib='reax',movieFrequency=10):
     ''' runword = keyword in gulp input
         can be 'md conv' 'md conp' 'opti conp'
         qite = iterative solve charges, norx = no charges '''
@@ -70,9 +74,12 @@ def write_gulp_in(A,runword='gradient qiterative nosymmetry conv verb debu',
        print('%s' %lib,file=finp)
     else:
        print('library %s' %lib,file=finp)
-
-    print('output movie xyz his.xyz',file=finp)
-    print('dump 100 restart.grs',file=finp)
+    
+    if output is None:
+       print('output movie {:d} xyz his.xyz'.format(movieFrequency),file=finp)
+    else:
+       print('output {:s}'.format(output),file=finp)
+    print('dump {:d} restart.grs'.format(restart),file=finp)
     print('#',file=finp)
     
     print('vectors',file=finp)
@@ -88,7 +95,6 @@ def write_gulp_in(A,runword='gradient qiterative nosymmetry conv verb debu',
         print(sp,'core',x[0],x[1],x[2],0.0,1.0,0.0,1,1,1,file=finp)
     finp.close()
 
-
 def get_reaxff_q(natom,fo='out'):
     q = []
     fout = open(fo,'r')
@@ -111,7 +117,6 @@ def get_reaxff_q(natom,fo='out'):
         q.append(float(l[2]))
     return q,ecoul,eself,evdw
 
-
 def get_reax_energy(fo='out'):
     fout = open(fo,'r')
     for line in fout.readlines():
@@ -146,7 +151,6 @@ def get_reax_energy(fo='out'):
     fout.close()
     return e,ebond,elp,eover,eunder,eang,epen,tconj,etor,fconj,evdw,ehb,ecl,esl
 
-
 def change_keyword(lib='reaxff_rdx'):
     system('cp inp-gulp inp-gulp.sample')
     fin  = open('inp-gulp','w')
@@ -162,7 +166,6 @@ def change_keyword(lib='reaxff_rdx'):
     fgin.close()
     fin.close()
 
-
 def get_md_results(out='gulp.out'):
     ''' get the result from the  gulp MD simulation '''
     flog = open(out,'r')
@@ -181,7 +184,6 @@ def get_md_results(out='gulp.out'):
     flog.close()
     return E,Epot,T,P
 
-
 def plot_md(E,Epot,T,P,show=True):
     plt.figure(figsize=(12,12))  
 
@@ -215,7 +217,6 @@ def plot_md(E,Epot,T,P,show=True):
        plt.savefig('MD_results.pdf') 
     plt.close()
 
-
 def plot_optimize(log='gulp.out'):
     ''' plot the energies or other from gulp log file'''
     flog = open(log,'r')
@@ -237,7 +238,6 @@ def plot_optimize(log='gulp.out'):
     plt.savefig('energy_vs_iteration.png') 
     plt.close()
 
-
 def get_lattice(inp='inp-gulp'):
     finp = open(inp,'r')
     il = 0
@@ -256,7 +256,6 @@ def get_lattice(inp='inp-gulp'):
     finp.close()
     return np.array(cell)
 
-
 def reaxyz(fxyz):
     # cell = get_lattice()
     f = open(fxyz,'r')
@@ -285,7 +284,6 @@ def reaxyz(fxyz):
                positions[nf][na][2] = float(l[3])
     return atom_name,positions,energies
 
-
 def xyztotraj(fxyz,checkMol=False,mode='w',scale=True):
     atom_name,positions,e = reaxyz(fxyz)
     cell = get_lattice()
@@ -310,7 +308,6 @@ def xyztotraj(fxyz,checkMol=False,mode='w',scale=True):
         del A
     his.close()
 
-
 def optl(T=350,gen='poscar.gen',mc=500):
     A = read(gen)
     # A = press_mol(A)
@@ -321,7 +318,6 @@ def optl(T=350,gen='poscar.gen',mc=500):
     system('/home/feng/gulp/gulp-5.0/Src/gulp<inp-gulp>gulp.out')
     xyztotraj('his.xyz',mode='w',scale=False)
 
-
 def opt(gen='siesta.traj',lib='reax',maxcyc=200,mode='w',
         keyword='opti conv qiterative',
         ncpu=1):
@@ -337,9 +333,8 @@ def opt(gen='siesta.traj',lib='reax',maxcyc=200,mode='w',
       system('gulp<inp-gulp>gulp.out')
     xyztotraj('his.xyz',mode='w')
 
-
 def nvt(atoms=None,gen='poscar.gen',T=350,time_step=0.1,tot_step=100,
-        keyword='md qiterative conv',
+        keyword='md qiterative conv',movieFrequency=10,
         ncpu=1,index=-1,mode='w',lib='reax'):
     if atoms is None:
        atoms = read(gen,index=index)
@@ -347,6 +342,7 @@ def nvt(atoms=None,gen='poscar.gen',T=350,time_step=0.1,tot_step=100,
                   T=T,
                   time_step=time_step,
                   tot_step=tot_step,
+                  movieFrequency=movieFrequency,
                   lib=lib)
     print('\n-  running gulp nvt ...')
     if ncpu>1:
@@ -355,7 +351,6 @@ def nvt(atoms=None,gen='poscar.gen',T=350,time_step=0.1,tot_step=100,
       system('gulp<inp-gulp>gulp.out')
     xyztotraj('his.xyz',mode=mode)
 
-
 def npt(T=350,time_step=0.1,tot_step=10.0,ncpu=1):
     A = read('packed.gen')
     write_gulp_in(A,runword='md conp qiterative',
@@ -369,7 +364,6 @@ def npt(T=350,time_step=0.1,tot_step=10.0,ncpu=1):
       system('gulp<inp-gulp>gulp.out')
     xyztotraj('his.xyz')
 
-
 def gulp(atoms=None,gen='poscar.gen',inp=None,out='gulp.out',ncpu=1):
     if atoms is None:
        atoms = read(gen)
@@ -380,9 +374,10 @@ def gulp(atoms=None,gen='poscar.gen',inp=None,out='gulp.out',ncpu=1):
     with open('gin','w') as finp:
          inp_ = inp.split(';')
          for line in inp_:
-             print(line,file=finp)
+             l_ = line.strip() 
+             print(l_,file=finp)
 
-         print('#',file=finp)
+         print(' ',file=finp)
          print('vectors',file=finp)
          cell = atoms.get_cell()
          for a in cell:
@@ -397,10 +392,102 @@ def gulp(atoms=None,gen='poscar.gen',inp=None,out='gulp.out',ncpu=1):
      
     print('-  running gulp ...')
     if ncpu>1:
-       system('mpirun -n %d gulp< %s > %s' %(ncpu,inp,out))
+       system('mpirun -n {:d} gulp< {:s} > {:s}'.format(ncpu,'gin',out))
     else:
-       system('gulp< %s > %s' %(inp,out))
-    xyztotraj('his.xyz')
+       system('gulp< {:s} > {:s}'.format('gin',out))
+    if isfile('his.xyz'):
+       xyztotraj('his.xyz')
     print('-  Job completed.')
 
+def jstolib(nn=False,jsfile='ffield.json',libfile='reax.lib'):
+    lf = open(jsfile,'r')
+    j = js.load(lf)
+    p_ = j['p']
+    m_ = j['m']
+    bo_layer_ = j['bo_layer']
+    mf_layer  = j['mf_layer']
+    lf.close()
 
+    spec,bonds,offd,angs,torp,hbs = init_bonds(p_)
+    write_lib(p_,spec,bonds,offd,angs,torp,hbs,m=m_,mf_layer=mf_layer,libfile=libfile)
+
+def init_bonds(p_):
+    spec,bonds,offd,angs,torp,hbs = [],[],[],[],[],[]
+    for key in p_:
+        # key = key.encode('raw_unicode_escape')
+        # print(key)
+        k = key.split('_')
+        if k[0]=='bo1':
+           bonds.append(k[1])
+        elif k[0]=='rosi':
+           kk = k[1].split('-')
+           # print(kk)
+           if len(kk)==2:
+              if kk[0]!=kk[1]:
+                 offd.append(k[1])
+           elif len(kk)==1:
+              spec.append(k[1])
+        elif k[0]=='theta0':
+           angs.append(k[1])
+        elif k[0]=='tor1':
+           torp.append(k[1])
+        elif k[0]=='rohb':
+           hbs.append(k[1])
+    return spec,bonds,offd,angs,torp,hbs
+
+def get_gulp_energy(atoms,ffild='reaxff_nn'):
+    write_gulp_in(atoms,runword='gradient nosymmetry conv qite verb',
+                  lib=ffield)
+    system('gulp<inp-gulp>out')
+
+    (e_,eb_,el_,eo_,eu_,ea_,ep_,
+    etc_,et_,ef_,ev_,ehb_,ecl_,esl_)= get_reax_energy(fo='out')
+    GE = {}
+    GE['ebond']        = eb_
+    GE['elonepair']    = el_
+    GE['eover']        = eo_
+    GE['eunder']       = eu_
+    GE['eangle']       = ea_
+    GE['econjugation'] = etc_
+    GE['epenalty']     = ep_
+    GE['etorsion']     = et_
+    GE['fconj']        = ef_
+    GE['evdw']         = ev_
+    GE['ecoul']        = ecl_
+    GE['ehb']          = ehb_
+    GE['eself']        = esl_
+    GE['E']            = e_
+    return GE
+
+def get_gulp_force_from_drv(fdrv):
+    forces = []
+    with open(fdrv,'r') as fdrv:
+         gl = 100
+         for i,line in enumerate(fdrv.readlines()):
+             if line.find('gradients cartesian eV/Ang')>=0: 
+                gl = i
+                natom = int(line.split()[3])
+             if i>gl and i<=gl+natom:
+                forces.append([float(line.split()[1]),float(line.split()[2]),float(line.split()[3])])
+    # print(forces)
+    return np.array(forces)
+
+def get_gulp_forces(images,traj='gulp_force.traj',ffield='reaxff_nn'):
+    ''' calculate the force by GULP '''
+    his  = TrajectoryWriter(traj,mode='w')
+    for i,atoms in enumerate(images):
+        write_gulp_in(atoms,runword='gradient nosymmetry conv qite verb',
+                      output='drv gulp',restart=1,
+                      lib=ffield)
+        system('gulp<inp-gulp>out')
+        forces = get_gulp_force_from_drv('gulp.drv')
+        #atoms.forces = forces
+        (e,eb_,el_,eo_,eu_,ea_,ep_,
+         etc_,et_,ef_,ev_,ehb_,ecl_,esl_)= get_reax_energy(fo='out')
+        #e = atoms.get_potential_energy()
+        calc = SinglePointCalculator(atoms,energy=e,
+                                     forces=-forces)
+
+        atoms.set_calculator(calc)
+        his.write(atoms=atoms)
+    his.close()
