@@ -12,61 +12,56 @@ from ..md.gulp import write_gulp_in,get_reax_energy
 import matplotlib.pyplot as plt
 
 
-def plot(e,Eb,Eu,Eo,El,Ea,Et,Ep,Etor,Ef,Ev,Ehb,Ec,show=False):
-    plt.figure(figsize=(15,12))    
-    plt.subplot(3,3,1)   
-    plt.plot(Eb,alpha=0.8,linestyle='-',color='b',label='Ebond')
-    plt.legend(loc='best',edgecolor='yellowgreen')
+def plot(e,Eb,Eu,Eo,El,Ea,Et,Ep,Etor,Ef,Ev,Ehb,Ec,figsize=(15,12),r=None,show=False):
+    total = 4
+    sEu = np.sum(np.square(Eu))
+    sEo = np.sum(np.square(Eo))
+    sEl = np.sum(np.square(El))
+    sEa = np.sum(np.square(Ea))
+    sEt = np.sum(np.square(Et))
+    sEp = np.sum(np.square(Ep))
+    sEtor = np.sum(np.square(Etor))
+    sEf = np.sum(np.square(Ef))
+    sEhb = np.sum(np.square(Ehb))
+    sEc = np.sum(np.square(Ec))
 
-    plt.subplot(3,3,2)   
-    plt.plot(Eo,alpha=0.8,linestyle='-',color='b',label='Eover')
-    plt.legend(loc='best',edgecolor='yellowgreen')
-
-    plt.subplot(3,3,3)   
-    plt.plot(Eu,alpha=0.8,linestyle='-',color='b',label='Eunder')
-    plt.legend(loc='best',edgecolor='yellowgreen')
-
-    plt.subplot(3,3,4)   
-    plt.plot(Ea,alpha=0.8,linestyle='-',color='b',label='Eang')
-    plt.legend(loc='best',edgecolor='yellowgreen')
+    for e_ in [sEu,sEo,sEl,sEa,sEt,sEp,sEtor,sEf,sEhb,sEc]:
+        if e_>0.001:
+           total += 1
     
-    plt.subplot(3,3,5)   
-    plt.plot(Ec,alpha=0.8,linestyle='-',color='b',label='Ecoul')
-    plt.legend(loc='best',edgecolor='yellowgreen')
+    if total<=6:
+       nrow = 3
+       ncol = 2
+    elif total<=9:
+       nrow = 3
+       ncol = 3
+    elif total<=12:
+       nrow = 4
+       ncol = 3  
+    else:
+       nrow = 4
+       ncol = 4  
+    Evb = np.array(Ev) + np.array(Eb)
+    Energies = [Eb,Ev,Evb,Eu,Eo,El,Ea,Et,Ep,Etor,Ef,Ehb,Ec,e]
+    labels   = ['Ebond','Evdw','Ebond & Evdw','Eunder','Eover',
+                'Elone','Eangle','Ethree','Epenialty','Etorsion',
+                'Efour','Ehbond','Ecoulomb','Etotal']
+    Es       = [1,1,1,sEu,sEo,sEl,sEa,sEt,sEp,sEtor,sEf,sEhb,sEc,1]
 
-    # plt.subplot(4,3,5)   
-    # plt.plot(Et,alpha=0.8,linestyle='-',color='b',label='Etcon')
-    # plt.legend(loc='best',edgecolor='yellowgreen')
-
-    # plt.subplot(4,3,6)   
-    # plt.plot(Ep,alpha=0.8,linestyle='-',color='b',label='Epen')
-    # plt.legend(loc='best',edgecolor='yellowgreen')
-
-    plt.subplot(3,3,6)   
-    plt.plot(Et,alpha=0.8,linestyle='-',color='b',label='Etor')
-    plt.legend(loc='best',edgecolor='yellowgreen')
-    
-    # plt.subplot(4,3,9)  
-    # plt.plot(Ef,alpha=0.8,linestyle='-',color='b',label='Efcon')
-    # plt.legend(loc='best',edgecolor='yellowgreen')
-    
-    Ebv = np.array(Ev) + np.array(Eb) + np.array(Eo) + np.array(Eu)
-    plt.subplot(3,3,7)   
-    plt.plot(Ehb,alpha=0.8,linestyle='-',color='b',label='Ehb')
-    plt.legend(loc='best',edgecolor='yellowgreen')
-
-    plt.subplot(3,3,8)   
-    plt.plot(Ev,alpha=0.8,linestyle='-',color='b',label='Evdw')
-    plt.legend(loc='best',edgecolor='yellowgreen')
-
-    plt.subplot(3,3,9)   
-    plt.plot(e,alpha=0.8,linestyle='-',color='b',label='Total Energy')
-    plt.legend(loc='best',edgecolor='yellowgreen')
-    plt.savefig('deb_energies.pdf')
-    if show: plt.show()
+    plt.figure(figsize=figsize) 
+    nf = 1
+    for es,e_,label in zip(Es,Energies,labels):
+        if es>0.001:
+           plt.subplot(nrow,ncol,nf)   
+           if r is None:
+              plt.plot(e_,alpha=0.8,linestyle='-',color='b',label=label)
+           else:
+              plt.plot(r,e_,alpha=0.8,linestyle='-',color='b',label=label)
+           plt.legend(loc='best',edgecolor='yellowgreen')
+           nf += 1
+    plt.show() if show else plt.savefig('deb_energies.pdf')
     plt.close()
 
-
 def deb_vdw(images,i=0,j=1,show=False):
     ir = IRFF_NP(atoms=images[0],
                  libfile='ffield.json',
@@ -108,13 +103,13 @@ def deb_vdw(images,i=0,j=1,show=False):
     plt.plot(eb,alpha=0.8,linewidth=2,linestyle='-',color='r',label=r'$E_{bond}$')
 
     plt.legend(loc='best',edgecolor='yellowgreen')
-    plt.savefig('deb_bo.pdf')
-    if show: plt.show()
+    plt.show() if show else plt.savefig('deb_bo.pdf')
+       
     plt.close()
     return eb,ev
 
 
-def deb_energy(images,atomi=0,atomj=1,debframe=[],show=False,nn=True,libfile='ffield.json'):
+def deb_energy(images,atomi=0,atomj=1,r_is_x=False,savecsv=False,show=False,nn=True,libfile='ffield.json',figsize=None):
     ir = IRFF_NP(atoms=images[0],
                  libfile=libfile,
                  nn=nn)
@@ -123,18 +118,20 @@ def deb_energy(images,atomi=0,atomj=1,debframe=[],show=False,nn=True,libfile='ff
     Eb,Ea,Ec,e = [],[],[],[]
     Ehb,Eo,Ev,Eu,El = [],[],[],[],[]
     Etor,Ef,Ep,Et = [],[],[],[]
-
-    fcsv = open('energies.csv','w')
-    csv_write = csv.writer(fcsv)
-    csv_write.writerow(['r','etotal','ebond','elone','eover','eunder','eangle',
-                        'econj','epen','etor',
-                        'efcon','evdw','ecoul','ehb'])
+    R=[]
+    
+    if savecsv:
+       fcsv = open('energies.csv','w')
+       csv_write = csv.writer(fcsv)
+       csv_write.writerow(['r','etotal','ebond','elone','eover','eunder','eangle',
+                            'econj','epen','etor',
+                            'efcon','evdw','ecoul','ehb'])
 
     for i_,atoms in enumerate(images):       
         ir.calculate(images[i_])
         # print('%d Energies: ' %i_,'%12.4f ' %ir.E, 'Ebd: %8.4f' %ir.ebond[0][1],'Ebd: %8.4f' %ir.ebond[2][3] )
         r  = ir.r[atomi][atomj]
-
+        R.append(r)
         Eb.append(ir.Ebond)
         Ea.append(ir.Eang)
         Eo.append(ir.Eover)
@@ -146,25 +143,53 @@ def deb_energy(images,atomi=0,atomj=1,debframe=[],show=False,nn=True,libfile='ff
         Ef.append(ir.Efcon)
         Etor.append(ir.Etor)
         Ehb.append(ir.Ehb)
-        Ec.append(ir.Ecoul)
+        Ecoul = ir.Ecoul if ir.Ecoul>0.00000001 else 0.0
+        Ec.append(Ecoul)
         e.append(ir.E)
-        csv_write.writerow([r,ir.E,ir.Ebond,ir.Elone,ir.Eover,ir.Eunder,ir.Eang, ir.Etcon,ir.Epen,ir.Etor,
+        if savecsv: csv_write.writerow([r,ir.E,ir.Ebond,ir.Elone,ir.Eover,ir.Eunder,ir.Eang, ir.Etcon,ir.Epen,ir.Etor,
                             ir.Efcon,ir.Evdw,ir.Ecoul,ir.Ehb])
-    fcsv.close()
-    plot(e,Eb,Eu,Eo,El,Ea,Et,Ep,Etor,Ef,Ev,Ehb,Ec,show=show)
+    if savecsv: fcsv.close()
+    if r_is_x:
+       plot(e,Eb,Eu,Eo,El,Ea,Et,Ep,Etor,Ef,Ev,Ehb,Ec,r=R,show=show,figsize=figsize)
+    else:
+       plot(e,Eb,Eu,Eo,El,Ea,Et,Ep,Etor,Ef,Ev,Ehb,Ec,r=None,show=show,figsize=figsize)
     return e
 
+def deb_bp(atoms,figsize=(8,6),libfile='ffield.json',nn=True):    
+    ir = IRFF_NP(atoms=atoms,
+                 libfile=libfile,
+                 nn=nn)
+    ir.calculate_Delta(atoms)
+    # print(ir.rcbo)
+    for i in range(ir.natom-1):
+        for j in range(i+1,ir.natom): 
+            if ir.bo0[i][j]<0.1:
+               continue 
+            ir.calculate(atoms)
+            r     = ir.r[i][j]
+            bsi   = ir.bop_si[i][j]
+            bpi   = ir.bop_pi[i][j]
+            bpp   = ir.bop_pp[i][j]
+            Bp    = ir.bop[i][j]      
+            B     = ir.bo0[i][j]  
+            Di    = ir.Deltap[i]-ir.bop[i][j]
+            Dj    = ir.Deltap[j]-ir.bop[i][j]
+ 
+            print('{:3d}{:2s}-{:3d}{:2s} r: {:5.3f} Bsi\': {:6.4f} Bpi\': {:6.4f} Bpp\': {:6.4f} '
+                  'Di: {:5.3f} B\': {:6.4f} Dj: {:5.6f} '
+                  'B: {:6.4f} '.format(i,ir.atom_name[i],j,ir.atom_name[j],r,
+                  bsi,bpi,bpp,Di,Bp,Dj,B))   
 
 def deb_bo(images,i=0,j=1,figsize=(8,6),print_=False,show=False,more=False,
-           fluctuation=False,u=1.5,l=1.35,libfile='ffield.json',
+           fluctuation=False,delta=False,u=1.5,l=1.35,libfile='ffield.json',
            nn=True,debug=False,bo_p=0,x_distance=False):
     r,bopsi,boppi,boppp,bo0,bo1,eb,esi = [],[],[],[],[],[],[],[]
-    
-    fcsv_ = open('bo.csv','w')
-    csv_write = csv.writer(fcsv_)
-    csv_write.writerow(['r','bosi0','bopi0','bopp0','bosi1','bopi1','bopp1',
-                        'bosi2','bopi2','bopp2',
-                        'bo0','bo1','esi','eb'])
+    Di,Dj = [],[]
+   #  fcsv_ = open('bo.csv','w')
+   #  csv_write = csv.writer(fcsv_)
+   #  csv_write.writerow(['r','bosi0','bopi0','bopp0','bosi1','bopi1','bopp1',
+   #                      'bosi2','bopi2','bopp2',
+   #                      'bo0','bo1','esi','eb'])
     
     ir = IRFF_NP(atoms=images[0],
                  libfile=libfile,
@@ -190,11 +215,13 @@ def deb_bo(images,i=0,j=1,figsize=(8,6),print_=False,show=False,more=False,
         bo1.append(ir.bo0[i][j])  
         eb.append(ir.ebond[i][j])   
         esi.append(ir.esi[i][j])
+        Di.append(ir.Deltap[i]-ir.bop[i][j])
+        Dj.append(ir.Deltap[j]-ir.bop[i][j])
         # N.append(ir.N[i][j])
-        csv_write.writerow([ir.r[i][j],ir.eterm1[i][j],ir.eterm2[i][j],ir.eterm3[i][j],
-                            ir.bop_si[i][j],ir.bop_pi[i][j],ir.bop_pp[i][j],
-                            ir.bosi[i][j],ir.bopi[i][j],ir.bopp[i][j],
-                            ir.bop[i][j],ir.bo0[i][j],ir.esi[i][j],ir.ebond[i][j]])
+      #   csv_write.writerow([ir.r[i][j],ir.eterm1[i][j],ir.eterm2[i][j],ir.eterm3[i][j],
+      #                       ir.bop_si[i][j],ir.bop_pi[i][j],ir.bop_pp[i][j],
+      #                       ir.bosi[i][j],ir.bopi[i][j],ir.bopp[i][j],
+      #                       ir.bop[i][j],ir.bo0[i][j],ir.esi[i][j],ir.ebond[i][j]])
 
         if x_distance:
            r.append(ir.r[i][j]) 
@@ -240,10 +267,18 @@ def deb_bo(images,i=0,j=1,figsize=(8,6),print_=False,show=False,more=False,
                 bopsi[-1],boppi[-1],boppp[-1],
                 ir.esi[i][j],bopsi[-1]*l,bopsi[-1]*u,
                 ir.esi[i][j]*bopsi[0]/esi[0],ir.ebond[i][j])) 
+           elif delta:
+              print('{:3d}  r: {:6.4f} Di: {:6.4f} Dj: {:6.4f} '
+                    'bo0: {:6.4f} bo1: {:6.4f} '
+                    'e: {:6.6f}'.format(i_,
+                    r[-1],Di[-1],Dj[-1],bo0[-1],bo1[-1],ir.ebond[i][j])) 
            else:  
               print('{:3d}  r: {:6.4f} bosi: {:6.4f} bopi: {:6.4f} bopp: {:6.4f} '
-                    'bo0: {:6.4f} bo1: {:6.4f} e: {:6.6f}'.format(i_,
-                r[-1],bopsi[-1],boppi[-1],boppp[-1],bo0[-1],bo1[-1],ir.ebond[i][j]))  
+                   'bo0: {:6.4f} bo1: {:6.4f} '
+                   'bosi: {:6.4f} bopi: {:6.4f} bopp: {:6.4f} e: {:6.6f}'.format(i_,
+               r[-1],bopsi[-1],boppi[-1],boppp[-1],bo0[-1],bo1[-1],
+               ir.bosi[i][j],ir.bopi[i][j],ir.bopp[i][j],ir.ebond[i][j]))  
+ 
     emin_ = np.min(eb)
     eb = (emin_ - np.array(eb) )/emin_
 
@@ -261,15 +296,13 @@ def deb_bo(images,i=0,j=1,figsize=(8,6),print_=False,show=False,more=False,
        plt.plot(r,eb,alpha=0.8,linewidth=2,linestyle='-',color='indigo',label=r'$E_{bond}$ ($-E_{bond}/%4.2f$)' %-emin_)
        # plt.plot(r,esi,alpha=0.8,linewidth=2,linestyle='-',color='indigo',label=r'$E_{esi}$ ($E_{si}/%4.2f$)' %emx)
     plt.legend(loc='best',edgecolor='yellowgreen')
-    plt.savefig('deb_bo.pdf')
-    if show: plt.show()
+    plt.show() if show else plt.savefig('deb_bo.pdf')
     plt.close()
     # return r,eb
-    fcsv_.close()
+    #fcsv_.close()
 
-def deb_f(images,i=0,j=1,figsize=(8,6),print_=False,show=False,more=False,
-           fluctuation=False,u=1.5,l=1.35,
-           nn=True,debug=False,bo_p=0,x_distance=False):
+def deb_f(images,i=0,j=1,figsize=(8,6),print_=False,show=False,
+           nn=True,x_distance=False):
     r,bopsi,boppi,boppp,bo0,bo1,eb,esi = [],[],[],[],[],[],[],[]
     
     ir = IRFF_NP(atoms=images[0],
@@ -307,6 +340,16 @@ def deb_f(images,i=0,j=1,figsize=(8,6),print_=False,show=False,more=False,
     plt.close()
     # return r,eb
 
+def deb_delta(atoms,libfile='ffield.json',nn=True):    
+    ir = IRFF_NP(atoms=atoms,
+                 libfile=libfile,
+                 nn=nn)
+    #ir.calculate_Delta(atoms)
+    ir.calculate(atoms)
+
+    for i in range(ir.natom):
+      print('ID: {:4d} Delta\': {:6.4f}'.format(i,ir.Deltap[i]))   
+
 def deb_eang(images,ang=[0,1,2],figsize=(8,6),show=False,print_=False):
     i,j,k = ang
     ang_  = [k,j,i]
@@ -411,7 +454,7 @@ def deb_eover(images,i=0,j=1,figsize=(16,10),show=False,print_=True):
     plt.close()
 
 
-def deb_gulp_energy(images,atomi=0,atomj=1,ffield='reax',show=False):
+def deb_gulp_energy(images,atomi=0,atomj=1,libfile='reax',show=False):
     ''' test reax with GULP, and run validation-set'''
     fcsv = open('gulp.csv','w')
     csv_write = csv.writer(fcsv)
@@ -433,7 +476,7 @@ def deb_gulp_energy(images,atomi=0,atomj=1,ffield='reax',show=False):
         #           pbc=(1, 1, 1))
         # A = atoms irff_jax(A)
         write_gulp_in(atoms,runword='gradient nosymmetry conv qite verb',
-                      lib=ffield)
+                      lib=libfile)
         system('gulp<inp-gulp>out')
 
         e_,eb_,el_,eo_,eu_,ea_,ep_,etc_,et_,ef_,ev_,ehb_,ecl_,esl_= \
@@ -469,25 +512,36 @@ def deb_gulp_energy(images,atomi=0,atomj=1,ffield='reax',show=False):
     return GE['Total-Energy']
  
 
-def compare_dft_energy(images,show=False,mol=None):
-    ir = IRFF_NP(atoms=images[0],
-                 libfile='ffield.json',
-                 mol=mol,nn=True)
-    ir.calculate_Delta(images[0])
+def compare_dft_energy(images,atomi=None,atomj=None,show=False,nn=True):
+    ir = IRFF_NP(atoms=images[0],libfile='ffield.json',nn=nn)
+    R,e_dft,e = [],[],[]
+    if atomi is None or atomj is None:
+       r_is_x=False
+    else:
+       r_is_x=True
 
-    e_dft,e = [],[]
-    
     for i_,atoms in enumerate(images):  
         e_dft.append(atoms.get_potential_energy())
         ir.calculate(images[i_])
         e.append(ir.E)
-        print(e_dft[-1],e[-1])
-        # print('r: {:6.4f} e: {:6.4f}'.format(ir.r[0][1],Ev[-1]))
-        
+        # print(e_dft[-1],e[-1])
+        if r_is_x:
+           r  = ir.r[atomi][atomj]
+           R.append(r)
+
+    e_min = np.min(e_dft)
+    e_dft = np.array(e_dft) - e_min
+    e_min = np.min(e)
+    e     = np.array(e) - e_min
+
     plt.figure()     
     
-    plt.plot(e,alpha=0.8,linewidth=2,linestyle='-.',color='b',label=r'$E_{ReaxFF-MPNN}$')
-    plt.plot(e_dft,alpha=0.8,linewidth=2,linestyle='-',color='r',label=r'$E_{DFT}$')
+    if r_is_x:
+       plt.plot(R,e,alpha=0.8,linewidth=2,linestyle='-.',color='b',label=r'$E_{ReaxFF-nn}$')
+       plt.plot(R,e_dft,alpha=0.8,linewidth=2,linestyle='-',color='r',label=r'$E_{DFT}$')
+    else:
+       plt.plot(e,alpha=0.8,linewidth=2,linestyle='-.',color='b',label=r'$E_{ReaxFF-nn}$')
+       plt.plot(e_dft,alpha=0.8,linewidth=2,linestyle='-',color='r',label=r'$E_{DFT}$')
 
     plt.legend(loc='best',edgecolor='yellowgreen')
     # plt.savefig('compare_dft.pdf')
@@ -495,6 +549,85 @@ def compare_dft_energy(images,show=False,mol=None):
     plt.close()
     return e
 
+def get_theta(atoms,figsize=(8,6),show=False,print_=False):
+    ir = IRFF_NP(atoms=atoms,
+                 libfile='ffield.json',
+                 nn=True)
+    ir.calculate(atoms)
+     
+    for a,angle in enumerate(ir.angs):  
+        i,j,k = angle
+        print('{:3d} {:3d} {:3d} {:3d}  {:6.4f}  {:6.4f} Dpi: {:6.4f} SBO: {:6.4f} pbo: {:6.4f} SBO3: {:6.4f}'.format(a,
+                     i,j,k,
+                     ir.thet0[a],ir.theta[a],ir.sbo[a],ir.SBO[a],ir.pbo[a],
+                     ir.SBO3[a])) # self.thet0-self.theta
+                     
+def deb_thet(images,ang=[0,1,2],figsize=(8,6),show=False,print_=False,dFrame=[]):
+    i,j,k = ang
+    ang_  = [k,j,i]
+    a     = 0
+    found = False
+    eang,ecoa,epen = [],[],[]
+    e,d,s = [],[],[]
+    
+    ir = IRFF_NP(atoms=images[0],
+                 libfile='ffield.json',
+                 nn=True)
+    ir.calculate_Delta(images[0])
+     
+    for i_,atoms in enumerate(images):       
+        ir.calculate(atoms)
+        
+        for na,angle in enumerate(ir.angs):  
+            ii,j_,k_ = angle
+            # print(angle)
+            if (ii==i and j_==j and k_==k) or (ii==k and j_==j and k_==i):
+               a = na
+               found = True
+               # print('find: ',a)
+        if not found:
+           print('Error: no angle found for',ang,angle)
+        eang.append(ir.Eang)    
+        ecoa.append(ir.Etcon)
+        epen.append(ir.Epen)
+        if i_ in dFrame:
+           e.append(ir.eang)
+           d.append(ir.dang)
+           s.append(ir.sbo)
+           # print(ir.Eang)
+        # if print_:
+           # for a,angle in enumerate(ir.angs): 
+           # if i_ in dFrame:
+        print('{:3d}  {:6.4f}  {:6.4f} Dpi: {:6.4f} SBO: {:6.4f} SBO3: {:6.4f} f7: {:6.4f} f8: {:6.4f} thet: {:6.4f} Eang: {:6.4f}'.format(i_,
+                     ir.thet0[a],ir.theta[a],ir.Dpi[j],ir.SBO[a],ir.SBO3[a],
+                     # ir.nlp[j],ir.SBO3[a],
+                     ir.f_7[a],ir.f_8[a], 
+                     ir.thet[a], 
+                     ir.eang[a])) # self.thet0-self.theta
+
+    if print_:
+       for a_,angle in enumerate(ir.angs):
+           print('id: {:3d}  {:3d}  {:3d}  {:3d}'.format(a_,
+                  angle[0],angle[1],angle[2]),end=' ')
+           print('Eang:',end=' ')
+           for e_ in e:
+               print(' {:6.4f} '.format(e_[a_]),end=' ')
+           print('Delta:',end=' ')
+           for e_ in d:
+               print('{:6.4f} '.format(e_[a_]),end=' ')
+           print('sbo:',end=' ')
+           for e_ in s:
+               print('{:6.4f} '.format(e_[a_]),end=' ')
+           print(' ')
+
+    plt.figure(figsize=figsize)     
+    plt.plot(eang,alpha=0.8,linewidth=2,linestyle='-',color='r',label=r'$E_{ang}$')# ($-E_{ang}/{:4.2f}$)'.format(ang_m))
+    # plt.plot(ecoa,alpha=0.8,linewidth=2,linestyle='-',color='indigo',label=r'$E_{coa}$') # ($E_{coa}/%4.2f$)' %emx)
+    # plt.plot(epen,alpha=0.8,linewidth=2,linestyle='-',color='b',label=r'$E_{pen}$') # ($E_{pen}/%4.2f$)' %eox)
+    plt.legend(loc='best',edgecolor='yellowgreen')
+    plt.savefig('deb_ang.pdf')
+    if show: plt.show()
+    plt.close()
 
 ## compare the total energy with DFT energy
 
